/**
  ******************************************************************************
  * @file board_config.h
  * @brief Header for board_config.c - Constants for registers'values
  * @author STMicroelectronics - MMY Application Team 
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
	
	
#ifndef __BOARD_CONFIG_H
#define __BOARD_CONFIG_H

// Includes ------------------------------------------------------------------
// Same level


//Lower level
#include "stm8l15x.h"

#include "mono_lcd.h"
#include "delay.h"

// Uncomment the line below in order to use the external EEPROM function. The function present in the iso15693 command sends several read multiple block command and then store the data into the external EEPROM  using I2C and DMA feature.
//#define USE_EXTERN_EEPROM_FUNCTION 

// Uncomment the line below in order to use the RTC peripheral in order to wake up the MCU periodically while the CR95HF is turned into low power mode
 #define USE_RTC_CLOCK_FUNCTION



// those constants allow to choose the board used in order witch on the LED and use the joystick
// -----------------------------------------------------------------------
// Change the constant below to define the board used
#define BOARD_SELECTED_EVALBOARD
//#define BOARD_SELECTED_DISCOVERYBOARD
// -----------------------------------------------------------------------


// For communication between MCU and CR95HF -------------------------------- 

//--------------------------------------------------------------------------
// For communication method detection test GPIOB3 is used to poll  SSI0 on the CR95HF board
// PB3  				Input			Floating			No interrupt
#define SSIO_TEST_GPIO_PORT 		GPIOB
#define SSIO_TEST_GPIO_PIN 			0x08


// For SPI communication -------------------------------

// PB7: MISO  		Input			Floating			No interrupt
#define SPI_MISO_GPIO_PORT 		GPIOB
#define SPI_MISO_GPIO_PIN 		0x80

// PB6: MOSI 			Output		Push-pull			10 MHz
#define SPI_MOSI_GPIO_PORT 		GPIOB
#define SPI_MOSI_GPIO_PIN 		0x40

// PB5: SCK 			Output		Push-pull			10 MHz
#define SPI_SCK_GPIO_PORT 		GPIOB
#define SPI_SCK_GPIO_PIN 			0x20

// PB4: NSS 			Output		Open-Drain		2 MHz
#define SPI_NSS_GPIO_PORT 		GPIOB
#define SPI_NSS_GPIO_PIN 			0x10

// PC2: IRQ_in 		Input			Pull-up				No interrupt (MCU point of view)
#define SPI_IRQ_IN_GPIO_PORT 		GPIOC
#define SPI_IRQ_IN_GPIO_PIN 			0x04

// PC3: IRQ_out 	Output		OpenDrain			2 MHz   (MCU point of view)
#define SPI_IRQ_OUT_GPIO_PORT 		GPIOC
#define SPI_IRQ_OUT_GPIO_PIN 			0x08



//---------------------------------------------------------
// For UART communication 

// PB4: NSS 			Output		Open-Drain		2 MHz
#define UART_NSS_GPIO_PORT 		GPIOB
#define UART_NSS_GPIO_PIN 			0x10

// PC2: UART_RX 		Input			Pull-up				No interrupt (MCU point of view)
#define UART_RX_GPIO_PORT 		GPIOC
#define UART_RX_GPIO_PIN 			0x04

// PC3: UART_TX 	Output		Push-pull			10 MHz   (MCU point of view)
#define UART_TX_GPIO_PORT 		GPIOC
#define UART_TX_GPIO_PIN 			0x08


//-------------------------------------------------------------------


#define MCU_DEFAULT_BAUD_RATE			57600 // Default baud rate of CR95HF


// For I2C commnication with external EEPROM
#define sEE_I2C                          I2C1
#define sEE_I2C_CLK                      CLK_Peripheral_I2C1
#define sEE_I2C_DR_Address               ((uint16_t)0x005216)

// DMA  For I2C communication with external EEPROM
#define sEE_I2C_DMA                      DMA1
#define sEE_I2C_DMA_CHANNEL_TX           DMA1_Channel3
#define sEE_I2C_DMA_CHANNEL_RX           DMA1_Channel0

//----------------------------------------------------------------------------



//--------------------------------------------------------------------------------
// For Eval board
//----------------------------------------------------------------------------

// Pin settings for LCD -----------------------------------------------

// PI3: LCD_MISO 	Input		floating			No interrupt   
#define LCD_MISO_GPIO_PORT 		GPIOI
#define LCD_MISO_GPIO_PIN 			0x08

// PI2: LCD_MOSI 	Output	Pushpull			2 MHz   
#define LCD_MOSI_GPIO_PORT 		GPIOI
#define LCD_MOSI_GPIO_PIN 			0x04

// PI1: LCD_SCK 	Output	Pushpull			2 MHz   
#define LCD_SCK_GPIO_PORT 		GPIOI
#define LCD_SCK_GPIO_PIN 			0x02

// PF2: LCD_NSS 	Output	Pushpull			2 MHz   
#define LCD_NSS_GPIO_PORT 		GPIOF
#define LCD_NSS_GPIO_PIN 			0x04


// For the LED on the board---------------------------------------------

//  LED 1-4 are switched off Schematic:  5v|----|LED|----|MCU pin
//  GPIOH 0-3 are set as output for LED 1-4
// pin-0-3 are set as push pull

#define LED_1_GPIO_PORT 		GPIOH  // Green LED
#define LED_1_GPIO_PIN 			0x01
   
#define LED_2_GPIO_PORT 		GPIOH	// Orange LED
#define LED_2_GPIO_PIN 			0x02
  
#define LED_3_GPIO_PORT 		GPIOH // Red LED
#define LED_3_GPIO_PIN 			0x04

#define LED_4_GPIO_PORT 		GPIOH	// Blue LED
#define LED_4_GPIO_PIN 			0x08


// To use the joystick ----------------------------------------------------
// GPOIG 0-4 	Input 		floating 			no interrupt

#define JOYSTICK_GPIO_PORT			GPIOG   

#define JOYSTICK_UP_GPIO_PIN 			0x01// up button

#define JOYSTICK_DOWN_GPIO_PIN 			0x02// down button

#define JOYSTICK_LEFT_GPIO_PIN 			0x04// left button

#define JOYSTICK_RIGHT_GPIO_PIN 			0x08// right button

#define JOYSTICK_SEL_GPIO_PIN 			0x10// sel button



//----------------------------------------------------------------------------------
// I2C pins for external EEPROM
#define sEE_I2C_SCL_PIN                  GPIO_Pin_1       /* PC.01 */
#define sEE_I2C_SCL_GPIO_PORT            GPIOC       			/* GPIOC */

#define sEE_I2C_SDA_PIN                  GPIO_Pin_0       /* PC.00 */
#define sEE_I2C_SDA_GPIO_PORT            GPIOC            /* GPIOC */


//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// For discovery Board 
//---------------------------------------------------------------------------

// Pin settings for LED -----------------------------------------------
// LED are active in high state  GND|----|LED|----|MCU pin
// Output   Pushpull 2Mhz
#define D_LED_3_GPIO_PORT 		GPIOE		// Green LED
#define D_LED_3_GPIO_PIN 			0x80

#define D_LED_4_GPIO_PORT			GPIOC		// Blue LED
#define D_LED_4_GPIO_PIN 			0x80

//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
#define SPI_INTERFACE_SELECTED 		0x01 // SSI0(PB3)=1 
#define UART_INTERFACE_SELECTED 	0x00 // SSI0=0 

//--------------------------------------------------------------------------------


// Exported functions --------------------------------------------------------

u8 Board_Get_SPI_or_UART_Communication(void);

void Board_GPIO_Config(void);

void Board_SPI_LCD_init(void);

void Board_UART_CR95HF_init(void);

void Board_SPI_CR95HF_init(void);

void Board_MCU_EEPROM_Unlock_for_data_storage(void);

#ifdef USE_EXTERN_EEPROM_FUNCTION
	void Board_sEE_LowLevel_DeInit(void);

	void Board_sEE_LowLevel_Init(void);
	
#endif

void Board_Init_Board(void);

void Board_Init_Display(void);



#endif /*__BOARD_CONFIG_H */
