/**
  ******************************************************************************
  * @file board_led.c
  * @brief This file contains all the functions utilized to control the led.
  * @author STMicroelectronics - MMY Application Team
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
// Include ------------------------------------------------------------------
#include "board_led.h"

/**  
* @brief  	this function switches on the led in parameter, refer to board_led.h for the leds
* @param  	LED: the name of the led to switch on
* @retval		void : No return value 
*/
void LED_Switch_on(u8 LED)
{
// check the board
#ifdef BOARD_SELECTED_EVALBOARD
		GPIO_ResetBits(GPIOH,LED); // activate LED
	
#else // discovery board
	
	if(LED==BLUE_LED)
		GPIO_SetBits(D_LED_4_GPIO_PORT, D_LED_4_GPIO_PIN);// activate LED
	if(LED==GREEN_LED)
		GPIO_SetBits(D_LED_3_GPIO_PORT, D_LED_3_GPIO_PIN);// activate LED
	if(LED==ALL_LED)
	{
		GPIO_SetBits(D_LED_4_GPIO_PORT, D_LED_4_GPIO_PIN);// activate LED
		GPIO_SetBits(D_LED_3_GPIO_PORT, D_LED_3_GPIO_PIN);// activate LED
	}
#endif
}

/**  
* @brief  	this function switches off the led in parameter, refer to board_led.h for the leds
* @param  	LED: the name of the led to switch off
* @retval		void : No return value 
*/
void LED_Switch_off(u8 LED)
{
	// Check the board
#ifdef BOARD_SELECTED_EVALBOARD
		GPIO_SetBits(GPIOH, LED);// activate LED // deactivate  LED
		
#else
	
	if(LED==BLUE_LED)
		GPIOC->ODR &= ~ BLUE_LED_DISCOVERYBOARD;// activate LED
	if(LED==GREEN_LED)
		GPIOE->ODR &=~ GREEN_LED_DISCOVERYBOARD;// activate LED
	if(LED==ALL_LED)
	{
		GPIO_ResetBits(D_LED_4_GPIO_PORT, D_LED_4_GPIO_PIN);// deactivate LED
		GPIO_ResetBits(D_LED_3_GPIO_PORT, D_LED_3_GPIO_PIN);// deactivate LED
	}
#endif
}

