/**
  ******************************************************************************
  * @file cr95hf_driver.h
  * @brief Header for cr95hf_driver.c - constants for communication settings with cr95HF
  * @author STMicroelectronics - MMY Application Team
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
	
	
#ifndef __CR95HF_DRIVER_H
#define __CR95HF_DRIVER_H

// Includes ------------------------------------------------------------------
//Same level
#include "miscellaneous.h"
#include "board_config.h"
#include "delay.h"

// Lower level
#include "stm8l15x.h"

// Define method for waiting answer from CR95HF
#define WAITING_CR95HF_RESPONSE_METHOD WAITING_USING_IRQ
//WAITING_USING_IRQ


// Exported constants --------------------------------------------------------
// MODE for SPI Communications
#define CTRLBYTE_SEND		 		0x00 
#define CTRLBYTE_READ_DATA 	0x02 
#define CTRLBYTE_POLLING 		0x03
#define CTRLBYTE_RESET		 	0x01 

// Only bits 3 to 7 are important so the byte should be 00001xxx, by shifting LSB are cut
// By this way bit3 is bit 0 and value to chack is 0x01
#define POLLING_CR95HF_READY		0x01
#define POLLING_CR95HF_SHIFT		0x03 

// WAITING RESPONSE METHOD
#define WAITING_USING_POLLING	0x00 
#define WAITING_USING_IRQ			0x01 

// TIME LIMIT FOR WAITING TRANSMISSION OR RCEPETION COMPLETE FLAG AND IRQ_in INTERRUPT
#define DEFAULT_DELAY_BEFORE_TIMEOUT  30 // 30 ms
/* In ISO15693 Communication the tag has a maximum time of 20 ms to answer a command then the CR95HF has to prepare the answer for transmission (response code, data length) and computes the CRC residue to check the data and add it at the end of the frame with control byte. Depending on several parameters  and configuration this time may vary
the timeout value  may not be lower than 27 ms (M24LRXX get system info command, low datarate)
Note that for some commands transmitting large amount of data the delay could be longer, for example a read multiple blocks in ISO15693 protocol with 32 blocks in low datarate (6kbits/s) can take up to around 205ms to transmit 32 blocks with the security status byte therefore the delay ought to be adjusted.
*/ 
#define DELAY_BEFORE_TIMEOUT_220MS  220 // 220 ms
#define DELAY_BEFORE_TIMEOUT_250MS  250 // 250 ms

// Max number of read before FIFO is empty
#define MAX_NUMBER_FIFO_READ	0x0A

// Constants for communication exit between MCU and CR95HF
#define EXIT_COMMUNICATION_OK 			0x00
#define EXIT_COMMUNICATION_TIMEOUT	0xEC

#define ERROR_CHECK_VALUE  					0xEC // value to return if timeout occurs

#define RESET_COMMAND  0x01 // This constant is redefined in upper level in cr95hf_command.h as CTRLBYTE_RESET

// Exported functions --------------------------------------------------------

void CR95HF_SPIChipSelect(FunctionalState NewState);

void CR95HF_Send_Byte(u8 DataToSend);

u8 CR95HF_Read_Byte(void);

u8 CR95HF_SPISendReceive_Byte(u8 DataToSend);

u8 CR95HF_Wait_Answer(u8 Timeout);

void CR95HF_SPIReset (void);

void CR95HF_NSS_Interrupt(void);

void CR95HF_IRQ_in_Interrupt(void);

void CR95HF_SPI_Chip_Enable_Read_Byte_Mode(void);

void CR95HF_SPI_Chip_Enable_Send_Byte_Mode(void);

void CR95HF_SPI_Chip_Disable(void);


#endif /*__CR95HF_DRIVER_H */

