/**
  ******************************************************************************
  * @file mono_lcd.h
  * @brief This file contains LCD functions prototypes.
  * @author STMicroelectronics - MCD Application Team
  * @version V1.1.1
  * @date 06/05/2009
  ******************************************************************************
  *
  */

#ifndef __MONO_LCD_H
#define __MONO_LCD_H

// Includes ------------------------------------------------------------------
#include "stm8l15x.h"

//#define USE_FULL_FUNCTIONS // uncommment to access all LCD functions

// Exported constants --------------------------------------------------------

// LCD managed as 2 Lines of 15 characters (2Lines * 15Char) 
#define LCD_LINE_MAX_CHAR ((u8)15)
#define LCD_LINE1         ((u8)0x80)
#define LCD_LINE2         ((u8)0x90)

// LCD Commands 
#define COMMAND_TYPE       ((u8)0xF8)
#define DATA_TYPE          ((u8)0xFA)
#define SET_TEXT_MODE      ((u8)0x30) /* 8-Bits Interface, Normal inst., Text mode */
#define SET_EXTENDED_INST  ((u8)0x34)
#define SET_GRAPHIC_MODE   ((u8)0x36) /* 8-Bits Interface, Extended inst., Graphic mode */
#define DISPLAY_ON         ((u8)0x0C) /* Cursor and blink off */
#define DISPLAY_OFF        ((u8)0x08)
#define DISPLAY_CLR        ((u8)0x01)
#define ENTRY_MODE_SET_INC ((u8)0x06)

#define LCD_STARTLEFTCORNER 								0x00
#define LCD_STARTMIDDLE 										0x04
#define LCD_FIRST_DOT												0x06
#define LCD_THIRD_DOT												0x07
#define LCD_LOGOTYPE_LINE1_LEFT_CORNER 			0x80
#define LCD_LOGOTYPE_LINE1_RIGHT_CORNER			0x87
#define LCD_LOGOTYPE_LINE2_LEFT_CORNER			0x90
#define LCD_LOGOTYPE_LINE2_RIGHT_CORNER			0x97

#define LCD_NSS_PIN_GPIOF  			0x04 	// PF2					

// Exported macro ------------------------------------------------------------

// Exported functions --------------------------------------------------------
void LCD_ReadStatus(void);
void LCD_ChipSelect(FunctionalState NewState);
void LCD_SendByte(u8 DataType, u8 DataToSend);

#ifdef USE_FULL_FUNCTIONS
	void LCD_SendBuffer(u8 *pTxBuffer, u8 *pRxBuffer, u8 NumByte);
#endif

void LCD_Initialize(void);
void LCD_Clear(void);

#ifdef USE_FULL_FUNCTIONS
	void LCD_SetTextMode(void);
	void LCD_SetGraphicMode(void);
	void LCD_ClearLine(u8 Line);
#endif

void LCD_SetCursorPos(u8 Line, u8 Offset);
void LCD_PrintChar(u8 Ascii);
void LCD_PrintString(u8 Line, FunctionalState AutoComplete, FunctionalState Append, u8 *ptr);
void LCD_DisplayCGRAM0(u8 address, u8 *ptrTable);
void LCD_DisplayCGRAM1(u8 address, u8 *ptrTable);
void LCD_DisplayLogo(u8 address);

#ifdef USE_FULL_FUNCTIONS
	void LCD_RollString(u8 Line, u8 *ptr, u16 speed);
#endif

void LCD_Delay(u16 nCount);
void LCD_PrintHex2(u8 Number);
void LCD_PrintDec7(u32 Number);
void LCD_PrintDec(u16 Number);
void LCD_Print(u8 *ptr);

#ifdef USE_FULL_LCD_FUNCTIONS
	void LCD_PrintMsg(u8 *ptr);
	void LCD_PrintHex1(u8 Number);
	void LCD_PrintHex3(u16 Number);
	void LCD_PrintBin2(u8 Number);
	void LCD_PrintBin4(u8 Number);
#endif



#endif /* __MONO_LCD_H */
