/**
  ******************************************************************************
  * @file cr95hf_structure.h
  * @brief This file contains the structures dedicated to the communication with CR9HF
  * @author STMicroelectronics - MMY Application Team
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
#ifndef __CR95HF_STRUCTURE_H
#define __CR95HF_STRUCTURE_H

#include "miscellaneous.h"

// Constants for the structures below
// Structures DATAlENGTH
#define MAX_DATARECEIVED_LENGTH 0x13 // limited to 19 bytes for iso15693 get sys info LRiS64K
#define MAX_DATASENT_LENGTH		 	0x10 // 16 bytes max ( used for Write kill ISO15693 Lrixk product)
#define MAX_PARAMETERS_LENGTH 	0x03


//-------------------------------------------------------
// Structure containing data and lenght of them
typedef struct 
{
	u8 cmd_code;/*!< Code of the command to send*/
	u8 data_length;/*!< length of the following data (LEN field) */
	u8 data[MAX_DATASENT_LENGTH]; /*!< Data to transmit, limited to MAX_DATASENT_LENGTH*/
	
}CR95HF_CommandSending;

// Structure containing the entire response provided by the CR95HF
typedef struct 
{
	u8 resp_code;/*!< Response code transmitted by the CR95HF  */
	u8 data_length;/*!< Length of the following data (LEN field) */
	u8 data[MAX_DATARECEIVED_LENGTH];/*!< Data to transmit, limited to MAX_DATARECEIVED_LENGTH*/ 
}CR95HF_ResponseReceiving;

// Structure containing the last protocol select i.e: the protocol in use and the paramaters associated, while there is no other protocol select command the protocol and parameters remain the same
typedef struct
{
	u8 protocol_in_use;/*!< Protocol in used set by the last protocol select command */
	u8 parameters_length;/*!< Length of the parameters, depending on the protocol */
	u8 parameters[MAX_PARAMETERS_LENGTH];/*!< parameters associated to the protocol,
	parameters may use up to 3 Bytes */
}CR95HF_Protocol_Config;


// Structure containing various parameters concerning the idle command. For instance this allows user to know the how to wake up the CR95HF if it is not in ready mode, the values set by the Tag detection calibration  procedure for tag detecting and the last reason of wake up of the CR95HF
typedef struct
{
	u8 wuFlags;/*!< To determine what can wake up CR95HF from Idle mode ( Interrupt, tag, detection, timer...)*/
	u8 dacDataL;/*!< Low comparing value for tag detection */
	u8 dacDataH;/*!< High comparing value for tag detection*/
	u8 last_reason_of_wakeup;/*!< Contains value describing the reason of the last idle mode exit*/
}CR95HF_Idle_Config;

	
	#endif /*__CR95HF_STRUCTURE_H */

