/**
  ******************************************************************************
  * @file iso15693_structure.h
  * @brief This file contains the structure dedicated to the iso15693 Tag
  * @author STMicroelectronics - MMY Application Team
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
#ifndef __ISO15693_STRUCTURE_H
#define __ISO15693_STRUCTURE_H

// lower level
#include "miscellaneous.h"
#include "delay.h"

// Constants for the structure below

#define ISO15693_STRUCT_INITVALUE						0xEC
#define ISO15693_DATA_RECEIVED_MAX_SIZE 		0x14 // 19 bytes
#define ISO15693_MEMSIZE_WORD_BYTE_SIZE			0x02// Mem size is described on two byte for a full iso15693 contactless tag
#define ISO15693_MEMSIZE_MAX_WORD_BYTE_SIZE 0x03
#define ISO15693_UID_SIZE										0x08

// ISO_15693_Command_Structure --------------------------------------------------------

typedef struct 
{
	u8 command_ok;/*!< The command and the response are ok, the structure is ok */ 
	u8 error_code;/*!<This field contains eventual error encountered by the contactless tag while performing the command. Also if the CR95HF sendRcv command fails to transmit the command or receive the contactless answer,the error code is stored here.*/
	u8 response_flags;/*!< The response flag provided by the Tag*/
	u8 dataField[ISO15693_DATA_RECEIVED_MAX_SIZE]; /*!< the whole data transmitted by the contactless tag*/
	u8 receivedDataLength; /*!< length of data transmitted by the Tag*/
	u8 UID_Tag[ISO15693_UID_SIZE];/*!< UID of the Tag with UID[0]= LSB UID[7]=MSB*/
	u8 AFI_Tag;/*!<AFI of the contactless tag*/
	u8 DSFID_Tag;	/*!<DSFID of the contactless tag*/
	u8 ICRef; /*!< ICRef of the contactless Tag */
	u8 memSize[ISO15693_MEMSIZE_MAX_WORD_BYTE_SIZE];/*!<MemSize of the contactless Tag
	 [0]= number of blocks [1]= number of bytes in a block
	  MemSize is on two bytes for iso15693 but could be on more for other products (thanks to IC REF this could be determined) limited  to 3 bytes.*/
	// MemSize is initialized in order to detect when a Get System Info command fills in the fields with valid data

}ISO15693_Tag;
	
	#endif /*__ISO15693_STRUCTURE_H */

