/**
  ******************************************************************************
  * @file    delay.h
  * @author  Microcontroller Division
  * @version V1.2.0
  * @date    09/2010
  * @brief   delay functions header
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2010 STMicroelectronics</center></h2>
  */
#include "stm8l15x.h"


#define WAIT_10us 		1			// to be used with delay_10us function
#define WAIT_100us 		10		// to be used with delay_10us function
#define WAIT_310us 		31		// to be used with delay_10us function ( inventory 16 slots)
#define WAIT_1ms			100			// to be used with delay_10us function

#define WAIT_3ms			3			// to be used with delay_ms function
#define WAIT_6ms			6			// to be used with delay_ms function
#define WAIT_10ms			10			// to be used with delay_ms function
#define WAIT_20ms			20			// to be used with delay_ms function
#define WAIT_500ms 		500 	// to be used with delay_ms function
#define WAIT_1s 			1000 	// to be used with delay_ms function

#define INITFREQ						0 			// Frequency is on 32bits
#define BASE_1ms						1000 		// Period is 1 ms i.e: 1/1000Hz 
#define BASE_10us						100000 	// Period is 1 ms i.e: 1/100kHz 
#define PRESCALER_1ms 				6			// To divide frequency clock by 2^6 
#define PRESCALER_1ms_VALUE		64		// Decimal value of the prescaler

// For Timer 2
#define TIM2_PERIOD  (uint8_t) 7

// Functions -----------------------------------------------

void delay_ms(u16 n_ms);

void delay_10us_base(u16 n_10us);

void Init_Timers_Values(void);

void Set_TimeOut(__IO uint32_t nTime);

void Reset_TimeOut(void);

u8 Get_Remaining_Time_before_TimeOut( void);

void Timing_TimeOut_Decrement(void);

void Timing_TimeOut_Init(void);



