/**
  ******************************************************************************
  * @file miscellaneous.h
  * @brief Header for miscellaneous.h - constant for ICREf
  * @author STMicroelectronics - MMY Application Team
  * @version V0.1
  * @date 
  ******************************************************************************
  *
  */
	
	
#ifndef __MISCELLANEOUS_H
#define __MISCELLANEOUS_H

// Lower level
#include "stm8l15x.h"
#include "stm8_eval_i2c_ee.h"

// Constants
#define BYTE_SIZE			0x08
#define SIZE_1KBITS_BYTES		0x80// 128 bytes=1Kbits

// Shifting ( LSB = bit 0)
#define SHIFT_BIT1			0x01
#define	SHIFT_BIT2			0x02 
#define SHIFT_BIT3			0x03
#define SHIFT_BIT4			0x04 
#define SHIFT_BIT5			0x05
#define SHIFT_BIT6			0x06 
#define SHIFT_BIT7			0x07
#define SHIFT_BYTE			0x08

#define SHIFT_BYTE_TO_BIT  	0x03 // multiply by 8 ie shift of 3
#define SHIFT_BIT_TO_KILO		0x0A // divide by 1024, ie: shift of 10

// Global constants
#define TRUE_CODE				0x00
#define FALSE_CODE			0x01
#define BIT_SET					0x01
#define	INIT_VALUE			0x00
#define	INIT_VALUE_u16	0x0000
#define	INIT_VALUE_u32	0x00000000
#define DATALENGTH_NULL	0x00

// Neutral elements for logic operations
#define NEUTRAL_ELEMENT_OR 			0x00
#define NEUTRAL_ELEMENT_AND 		0xFF

// EEPROM Memory for data storage
// INTERNAL EEPROM ----------------------------------------------------------
// EEPROM is used to store heavy amount of data for example commands sending or receiving  more than MAX data defined in structure files. To avoid creation of heavy array the EEPROM is used to store those data.
// 1kbits bytes are reserved for data transmission ( read or write mulitple block)
//128 bytes are reserved to store UIDs received during an inventory 16 slots ( 16x 8 bytes per UID)
#define EEPROM_UID_PHYSICAL_ADDRESS		FLASH_DATA_EEPROM_START_PHYSICAL_ADDRESS // + 128 bytes 0x00107F
#define EEPROM_DATA_PHYSICAL_ADDRESS   0x001080// + 128 bytes 0x0011FF

#define EEPROM_END					FLASH_DATA_EEPROM_END_PHYSICAL_ADDRESS
#define EEPROM_WORD_SIZE  0x04
//---------------------------------------------------------------------------
//EXTERNAL EEPROM -----------------------------------------------------------
#define EXTERNAL_EEPROM_SIZE_BITS		65536  //  64Kbits
#define EXTERNAL_EEPROM_SIZE_BYTES		8192  //  8KBytes
#define EXTERNAL_EEPROM_FIRST_ADDRESS 0x0000
//---------------------------------------------------------------------------
// ICREF AND PRODUCT MEMSIZE
/*STMicroelectronics products are listed below. Each time you had a product with its ICref, you should add:
1-  The ICRef to identify the product
2- 	The size of the word describing the memory size
3- 	The text product name to display 
4- 	Add the code below to the functions :Compare_IcRef_Return_Memsize(u8 Icref) and Compare_ICRef_Return_Size_of_Block(u8 Icref) in miscellaneous.c
	if(Icref==NEW_PRODUCT)
		nb_byte=NEW_PRODUCT_MEMSIZE_WORD_SIZE;
5- 	Add the code below to the function : in miscellaneous.c
	if(Icref==NEW_PRODUCT)
		nb_byte=NEW_PRODUCT_BLOCK_SIZE ;
/*
_____________________________________________________________________
 ICRef  ||       Product Name    	|| 	MemSize   || Byte for MemSize	|
---------------------------------------------------------------------
0x40		||					LRI1k					||   1kbits		||        2					|
---------------------------------------------------------------------
0x20		||					LRI2k					||   2kbits		||        2					|
---------------------------------------------------------------------
0x28		||					LRIS2k				||   2kbits		||        2					|
---------------------------------------------------------------------
0x44		||					LRiS64k				||	64kbits  	||				3					|
---------------------------------------------------------------------
0x2C		||					M24LR64-R			||	64kbits  	||				3					|
---------------------------------------------------------------------
0x4C		||					M24LRXX-E			||	XXkbits  	||				3					|
---------------------------------------------------------------------
0x4E		||					M24LR16-E			||	16kbits  	||				3					|
---------------------------------------------------------------------
____________________________________________________________________|
*/
// ICREF 
#define M24LR64_R		0x2C
#define M24LRXX_E		0x4C
#define M24LR16_E		0x4E
#define LRIS64K			0x44
#define LRIS2K			0x28
#define LRI2K				0x20
#define LRI1K				0x40

#define  LRIS64K_MEMSIZE_WORD_SIZE      0x03
#define  M24LR64_R_MEMSIZE_WORD_SIZE    0x03
#define  M24LRXX_E_MEMSIZE_WORD_SIZE    0x03
#define  M24LR16_E_MEMSIZE_WORD_SIZE    0x03
#define  LRI2K_MEMSIZE_WORD_SIZE        0x02
#define  LRIS2K_MEMSIZE_WORD_SIZE       0x02
#define  LRI1K_MEMSIZE_WORD_SIZE        0x02

#define  LRIS64K_BLOCK_SIZE      0x04
#define  M24LR64_R_BLOCK_SIZE    0x04
#define  M24LRXX_E_BLOCK_SIZE    0x04
#define  M24LR16_E_BLOCK_SIZE    0x04
#define  LRI2K_BLOCK_SIZE        0x04
#define  LRIS2K_BLOCK_SIZE       0x04
#define  LRI1K_BLOCK_SIZE        0x04

// PRODUCTS NAME          
#define LRIS64K_TEXT			(" ** LRIS64k **")
#define M24LR64_R_TEXT		("** M24LR64-R **")
#define M24LRXX_E_TEXT		("** M24LRXX-E **")
#define M24LR16_E_TEXT		("** M24LR16-E **")
#define LRI2K_TEXT				(" **  LRI2k  **")
#define LRIS2K_TEXT				(" **  LRIS2k **")
#define LRI1K_TEXT				(" **  LRI1k  **")

#define NO_ICREF_MATCHING 							0x00
#define MASK_ICREF_6MSB									0xFC // to retrieve only the 6 MSB bits for ICRef


//Functions

u8 ComputeByte(const u8 Bit7,const  u8 Bit6,const  u8 Bit5,const  u8 Bit4,const  u8 Bit3,const  u8 Bit2,const  u8 Bit1,const  u8 Bit0);

u8 Compare_IcRef_Return_Memsize(const u8 ICRef);

u8 Compare_ICRef_Return_Size_of_Block(const  u8 ICRef);

void Get_Data_From_EEPROM(u8* data_retrieved ,u32 Address, u8 Length)  ;

void Store_Data_Into_EEPROM(u32 Storing_Address, u8* Data_to_Store, u8 Length) ;


#ifdef BOARD_SELECTED_EVALBOARD

#ifdef USE_EXTERN_EEPROM_FUNCTION
void  Transfer_data_internal_to_external_EEPROM(u16 Dest_Address_Ext_EEPROM, u16 Src_Address_Int_EEPROM, u8 Size_of_data );

void  Transfer_data_external_EEPROM_to_internal_EEPROM(u16 Dest_Address_Int_EEPROM, u16 Src_Address_Ext_EEPROM, u8 Size_of_data);
#endif
#endif

#endif /*__MISCELLANEOUS_H*/


