/**
  ******************************************************************************
  * @file user_application.h
  * @brief Header for user_application.c
  * @author STMicroelectronics - MMY Application Team
  * @version V1.0
  * @date 20 09 2011
  ******************************************************************************
  *
  */
	
	
#ifndef __USER_APPLICATION_H
#define __USER_APPLICATION_H

// Includes ------------------------------------------------------------------
#include "cr95hf_command.h"
#include "board_led.h"


// constant for joystick state
// when pressed button level drops to low level, so when no button is pressed the GPIOG input data word is 0x1F
#define JOYSTICK_NO_BUTTON_PRESSED								0x1F  // GPIOG Input data register 1.1111
#define JOYSTICK_UPSTATE       										0x1E  // GPIOG Input data register 1.1110
#define JOYSTICK_DOWNSTATE    							 			0x1D	// GPIOG Input data register 1.1101
#define JOYSTICK_LEFTSTATE     										0x1B  // GPIOG Input data register 1.1011
#define JOYSTICK_RIGHTSTATE    										0x17  // GPIOG Input data register 1.0111
#define JOYSTICK_SELSTATE      										0x0F  // GPIOG Input data register 0.1111  

// Constant for the idle command
// For tag calibration test
#define IDLE_TEST_WUP_160ms												0x14 // 20xTref( 8ms@32kHz)
#define IDLE_TEST_MAXSLEEP_32_TRIALS							0x1F // 31+1 trials( max value)
// For mode 1&2
#define IDLE_WAKE_UP_IRQ_IN_TAG_DETECTION					0x0A
#define IDLE_TEST_WUP_32ms												0x04 // 4xTref( 8ms@32kHz)
// for mode 3
#define IDLE_WAKE_UP_IRQ_IN_TAG_DETECTION_TIMER		0x0B
#define IDLE_TEST_WUP_256ms												0x20 // 32xTref( 8ms@32kHz)
#define IDLE_TEST_MAXSLEEP_20_TRIALS							0x13 // 19+1 trials

// Exported functions --------------------------------------------------------
void RunUserApplication (void);
static u8 GetJoystickState (void);
void Config_tag_detection(void);
u8 Select_Protocol_ISO15693(void);
void Wait_Display(u16 Time);

#endif /*__USER_AAPPLICATION_H*/


