/**
  ******************************************************************************
  * @file user_application.c
  * @brief This contains the functions called in the demo mode using the joystick
  * @author STMicroelectronics - MMY Application Team
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
// Includes ------------------------------------------------------------------
#include "user_application.h"

// Uncomment one of the line below to choose one of the three low power mode proposed here
//#define STM8L_LOW_POWER_MODE_WFI
#define STM8L_LOW_POWER_MODE_HALT_CR95HF_TIMER
//#define STM8L_LOW_POWER_MODE_HALT
/* ========================Description of the three low power modes ========================
  Wait For Interrupt mode ---------------------------------------------------
			In this mode the CPU is deactived, to ensure that the MCU will wake up set a RTC interrupt every 5 seconds 
	then if there is no tag seen the MCU wakes up the CR95HF, checks the communication and turns again the CR95HF into the Idle mode. 
	Uncomment the line #define USE_RTC_CLOCK_FUNCTION
	into the board_config.h file to enable the function configuring the RTC clock and settings

	Halt mode ----------------------------------------------------------
			In Halt Mode  very low power consumption is achieved, the main clock and all peripherals are  switch off. Only  Interrupt on I/O pin or reset can wake up the STM8L. In this mode only a tag detection or a pressur on the reset button can wake up the system. 

 Halt CR95HF timer mode ----------------------------------------------------------
	In this mode the STM8L is tuned into halt mmode and the the CR95HF wakes up periodically ( around every 5 seconds)
	This wakes up the STM8L which try to communicate with the CR95HF, and retrieve what caused the waking up (tag detection or timer). If timer caused the waking up then the STM8L send again the Idle command else a tag has been detected, the field is turned on in ISO15693 and a communication is tried with the contactless tag. 

In summary :  

In all mode a tag detection wakes up the CR95HF which wakes up the STM8L
In mode 1 and 3 the system wakes up periodically,
In mode 1 the STM8L wakes up periodically, then wakes up the CR95HF and re send an Idle command turning again into low power mode.
In mode 3 the CR95HF wakes up periodically which wakes the STM8L. Then the STM8L determines what causes the waking up of the CR95HF and returns into low power if timer caused the event.
========================Description of the three low power  modes ========================
*/

// This structure  is retrieve in order to achieve the Idle command and the tag detection calibration.  
extern CR95HF_Idle_Config Idle_Config;
// This structure is retrieved from the main file in order to store the protocol settings
extern CR95HF_Protocol_Config Protocol_Settings; 
// This structure is retrieved from the main file in order to store the CR95HF answer
extern CR95HF_ResponseReceiving CR95HF_response;
// This variable is retrieved from the main file in order to store the protocol ISO15693 paramters
extern u8 ISO15693_ProtocolSelect_Parameters;


/**
* @brief This function runs the user's application according to the joystick state
* @param none
* @retval none
*/
void RunUserApplication (void )
{
	u8 ISO15693_ProtocolSelect_Parameters;
	u8 value;// to determine the joystick button pressed
	u8 deviceID_ctr;// counetr for looping
	u8 deviceID_char=INIT_VALUE; // incrementing counter to display data (string)
	u8 status=CR95HF_ERROR_CODE;
	u8 status_Tag_seen= FALSE_CODE; // Here TRUE_CODE=EXIT_COMMUNICATION_OK
	
	// Get the joystick state
	value = GetJoystickState ();
	// call the right function according to the state
	switch (value)
	{
			case JOYSTICK_UPSTATE://up
				// Send an IDN command and display the device ID.
				CR95HF_IDN (&CR95HF_response);
				// DEVICE ID DISPLAY
				LCD_Clear();// CLear the LCD screen
				LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"  Device ID :");
				// deviceID_ctr is used as a decrementing counter, it's placed at the end of the DEVICE ID  
				deviceID_ctr=(CR95HF_response.data_length-CR95HF_IDN_ROMCRC_CURSOR);
				LCD_SetCursorPos(LCD_LINE2, LCD_STARTLEFTCORNER);//LCD CUSOR IS SET AT LINE LEFT CORNER
				while(deviceID_ctr)
				{
					// Display the entire chain of the DEVICE ID
					LCD_PrintChar(CR95HF_response.data[deviceID_char++]);
					deviceID_ctr--;
				}
				Wait_Display(WAIT_1s);// Wait for 1 sec and refresh the LDC screen
					
			break;
			
			case JOYSTICK_DOWNSTATE: // down
				// Toggle the RF field 
				LED_Switch_off(BLUE_LED);// switch off Blue LED
				// if field is on, turn it off
				// Information about the protocol selected are stored into the Protocol_Settings structure
				if(Protocol_Settings.protocol_in_use!=CR95HF_PROT_SEL_FIELDOFF)
				{
					// turn off the field and update the Protocol_Settings structure
					status=CR95HF_FieldOff(&Protocol_Settings);
					LED_Switch_on(YELLOW_LED);// activate  yellow LED
					// Clear the LCD screen and display field off
					LCD_Clear();
					LCD_PrintString(LCD_LINE1,ENABLE,DISABLE," Field OFF .");
					Wait_Display(WAIT_1s);
			
				}
				else // turn field on with protocol ISO15693
				{
					// Call the Select_Protocol_ISO15693 which will call the protocol select function with particular parameters
					Select_Protocol_ISO15693();
					// Clear the LCD screen and display field on
					LCD_Clear();
					LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"  Field ON .)))");
					Wait_Display(WAIT_1s);
					LED_Switch_off(YELLOW_LED);// deactivate yellow LED
				}
				
			
			break;
			
			case JOYSTICK_LEFTSTATE:// left
			// A calibration of the tag detector is made in order to use the Idle mode and exit it when a tag is presented in the field.
			LED_Switch_off(BLUE_LED); // field is turned off by the procedure turn off the blue LED
			// Tag detector Calibration
			Config_tag_detection();
		
					
			break;
			
			case JOYSTICK_RIGHTSTATE:// right low power mode
					
			// The purpose of this mode is to lower down the consumption of the CR95HF and the STM8L. However, if a tag is presented into the operating volume, the CR95HF should be able to detect it and the system should wake up and enter in communication with the contactless tag 
			
			LED_Switch_off(ALL_LED);// consumpt 
			// Make some display to inform that the CR95HF will be turned into Idle mode
			LCD_Clear();
			LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"Turn into Idle");
			delay_ms(WAIT_500ms);
			LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"Search for tag");
			delay_ms(WAIT_1s);
			// Turn off the field  (Idle will do so anyway if this step is not done
			CR95HF_FieldOff(&Protocol_Settings);
			// Note that in UART the NSS pin should be set as a wake up condition instead of IRQ_in
			
/* ====================== First low power mode=======================================
 In this mode the CR95HF can wake up by detecting a contactless tag. A RTC interrupt is sent every 5 seconds. Thus if the CR95HF has not seen a tag during 5 seconds, the STM8Ls RTC clock wakes up the MCU which sends an Interrupt to wake up the CR95HF.
======================== First low power mode=====================================*/	 
#ifdef STM8L_LOW_POWER_MODE_WFI 

			LCD_PrintString(LCD_LINE2,ENABLE,DISABLE," Wait Mode #1");
			do
			{
				// Enable interrupt on the IRQ_in pin, when the CR95HF wakes up a interrupt is sent on the IRQ_in (MCU) 
				GPIO_Init(SPI_IRQ_IN_GPIO_PORT,SPI_IRQ_IN_GPIO_PIN ,GPIO_Mode_In_FL_IT );
				// Configure sensitivity (Falling edge) pin 2 (for PC2) 
				EXTI_SetPinSensitivity(EXTI_Pin_2,EXTI_Trigger_Falling);  
				// Send the command idle 
/*
				CR95HF_Idle(IDLE_WAKE_UP_IRQ_IN_TAG_DETECTION,
										CR95HF_DETECTOR_CALIBRATION_EnterCtrl_L,
										CR95HF_DETECTOR_CALIBRATION_EnterCtrl_H,
										CR95HF_DETECTOR_CALIBRATION_WUCtrl_L,
										CR95HF_DETECTOR_CALIBRATION_WUCtrl_H,
										CR95HF_IDLE_LEAVE_CTRL_L,
										CR95HF_IDLE_LEAVE_CTRL_H ,
										IDLE_TEST_WUP_32ms,
										CR95HF_IDLE_OSC_START,
										CR95HF_IDLE_DAC_START	,
										Idle_Config.dacDataL,
										Idle_Config.dacDataH, 
										CR95HF_IDLE_SWING_CNT,
										INIT_VALUE, 
										&Idle_Config);
*/
				CR95HF_TagDetecting (	CR95HF_IDLE_WAKEUP_NONE,
															IDLE_TEST_WUP_32ms,
															CR95HF_IDLE_LFOFREQ_32K ,
															Idle_Config.dacDataL,
															Idle_Config.dacDataH,
															CR95HF_IDLE_MAXSLEEP_NULL,
															&Idle_Config);
				// The CR95HF is turned into Idle mode
				// Set the timer to wakes up the STM8L
				RTC_SetWakeUpCounter(5);// 5 seconds
				RTC_WakeUpCmd(ENABLE);// Enable the RTC interrupt into 5 seconds
				// turn into WFI mode
				wfi();
				// Interrupt is detected but its cause (RTC or CR95HF) is not yet known 
				RTC_WakeUpCmd(DISABLE);// Disable the RTC clock 
				LED_Switch_on(YELLOW_LED);// activate yellow LED to make a blink
				EXTI_DeInit(); // deactive external interrupt
				// Reset IRQ_in pin with no interrupt
				GPIO_Init(SPI_IRQ_IN_GPIO_PORT,SPI_IRQ_IN_GPIO_PIN ,GPIO_Mode_In_PU_No_IT);
				// Try to receive a response from the CR95HF (if awake) 
				status_Tag_seen=CR95HF_Wait_Wake_up_from_Idle(&CR95HF_response,&Idle_Config,CR95HF_NUMBER_OF_WAITING_PERIOD);
				// test if a response was received
				if(status_Tag_seen!=EXIT_COMMUNICATION_OK)
				{
					// The RTC wakes up the MCU, the CR95HF is not in ready mode
					// send an interrupt in order to wake him up 
					CR95HF_Wake_up_CR95HF(&Idle_Config);
					status=CR95HF_Echo();// Test the communication
				}
				LED_Switch_off(YELLOW_LED);// switch off the yellow LED
			}
			while(status==CR95HF_SUCCESS_CODE && status_Tag_seen!=TRUE_CODE);
			// The loop ends if a tag is seen (CR95HF woke up alone) or if th MCU failed to wake up the CR95HF
#endif		
/* ====================== Second low power mode=======================================
	 In this mode the CR95HF can wake up by detecting a contactless tag. 
	 In this mode the STM8L can wake up by receiving a pulse on its IRQ_in ( CR95HF wakes up) or by pushing the reset button. Note that pressing this button will restart the MCU and wake up the CR95HF (starting procedure send an interrupt  on the IRQ_in of the CR95HF
 ====================== Second low power mode=======================================*/
#ifdef STM8L_LOW_POWER_MODE_HALT 

	LCD_PrintString(LCD_LINE2,ENABLE,DISABLE," Wait Mode #2");
/*	
	CR95HF_Idle(	IDLE_WAKE_UP_IRQ_IN_TAG_DETECTION,
								CR95HF_DETECTOR_CALIBRATION_EnterCtrl_L,
								CR95HF_DETECTOR_CALIBRATION_EnterCtrl_H,
								CR95HF_DETECTOR_CALIBRATION_WUCtrl_L,
								CR95HF_DETECTOR_CALIBRATION_WUCtrl_H,
								CR95HF_IDLE_LEAVE_CTRL_L,
								CR95HF_IDLE_LEAVE_CTRL_H ,
								IDLE_TEST_WUP_32ms, 
								CR95HF_IDLE_OSC_START,
								CR95HF_IDLE_DAC_START	,// Setting time before making measures
								Idle_Config.dacDataL,
								Idle_Config.dacDataH,// values for detection   
								CR95HF_IDLE_SWING_CNT,
								INIT_VALUE, // Number of pulses emitted and number of mesure (timer not activated)
								&Idle_Config);// struture to update with Wake up flags
*/
	CR95HF_TagDetecting (	CR95HF_IDLE_WAKEUP_IRQIN,
												IDLE_TEST_WUP_32ms,
												CR95HF_IDLE_LFOFREQ_32K ,
												Idle_Config.dacDataL,
												Idle_Config.dacDataH,
												CR95HF_IDLE_MAXSLEEP_NULL,
												&Idle_Config);
	
			// Turn the STM8L in halt mode
			halt();  
			// The only way out possible is a tag detection ( a reset returns to the begining of the application)
			// Retrieve the cause of the waking up ( tag detection) and resme communication  			
			status_Tag_seen=CR95HF_Wait_Wake_up_from_Idle(&CR95HF_response,&Idle_Config,CR95HF_NUMBER_OF_WAITING_PERIOD);
#endif
//------------------------------------------------------------------------------------


/* ====================== Third low power mode=======================================
In this mode the CR95HF can wake up by detecting a contactless tag or by its internal timer (5s). 
	 In this mode the STM8L is wake up by the CR95HF. the wause of the wake up is either a detecion of a contacless tag or the CR95HF timeout. After the wake up the STM8L checks the wause of the wake up.
========================= Third low power mode=====================================*/
#ifdef STM8L_LOW_POWER_MODE_HALT_CR95HF_TIMER
	// In this mode the CR95HF wakes up periodically thanks to its internal timer
			LCD_PrintString(LCD_LINE2,ENABLE,DISABLE," Wait Mode #3");
			do
			{
				// Enable interrupt on the IRQ_in pin 
				GPIO_Init(SPI_IRQ_IN_GPIO_PORT,SPI_IRQ_IN_GPIO_PIN ,GPIO_Mode_In_FL_IT );
				// Configure sensitivity (Falling edge) pin 2 (for PC2) 
				EXTI_SetPinSensitivity(EXTI_Pin_2,EXTI_Trigger_Falling);  
				// Send the Idle command
/*
				CR95HF_Idle(	IDLE_WAKE_UP_IRQ_IN_TAG_DETECTION_TIMER,
											CR95HF_DETECTOR_CALIBRATION_EnterCtrl_L,
											CR95HF_DETECTOR_CALIBRATION_EnterCtrl_H,
											CR95HF_DETECTOR_CALIBRATION_WUCtrl_L,
											CR95HF_DETECTOR_CALIBRATION_WUCtrl_H,
											CR95HF_IDLE_LEAVE_CTRL_L,
											CR95HF_IDLE_LEAVE_CTRL_H ,
											IDLE_TEST_WUP_256ms, 
											CR95HF_IDLE_OSC_START, CR95HF_IDLE_DAC_START	,
											Idle_Config.dacDataL, 
											Idle_Config.dacDataH,
											CR95HF_IDLE_SWING_CNT,
											IDLE_TEST_MAXSLEEP_20_TRIALS,
											&Idle_Config);
*/		
				CR95HF_TagDetecting (	CR95HF_IDLE_WAKEUP_IRQIN | CR95HF_IDLE_WAKEUP_TIMER,
															IDLE_TEST_WUP_256ms,
															CR95HF_IDLE_LFOFREQ_32K ,
															Idle_Config.dacDataL,
															Idle_Config.dacDataH,
															IDLE_TEST_MAXSLEEP_20_TRIALS,
															&Idle_Config);
												
				// turn into halt mode
				halt();
				// Either the CR95HF's internal timer wakes up it and by consequence the STM8L or a tag was seen
				LED_Switch_on(YELLOW_LED);// activate yellow LED to make a blink
				// Reset IRQ_in pin with no interrupt
				GPIO_Init(SPI_IRQ_IN_GPIO_PORT,SPI_IRQ_IN_GPIO_PIN ,GPIO_Mode_In_PU_No_IT);
				EXTI_DeInit();// disable external interrupt
				// Try to retrieve the reason of waking up
				status=CR95HF_Wait_Wake_up_from_Idle(&CR95HF_response,&Idle_Config,CR95HF_NUMBER_OF_WAITING_PERIOD);
				status=CR95HF_Echo();// try to communicate
				// Check if the CR95HF woke up because of a contactless tag
				status_Tag_seen=CR95HF_Is_Tag_Detected_Wake_up_Condition(Idle_Config.last_reason_of_wakeup);
				LED_Switch_off(YELLOW_LED);// switch off the yellow LED
			}
			while(status==CR95HF_SUCCESS_CODE && status_Tag_seen!=TRUE_CODE);
			
#endif
//------------------------------------------------------------------------------------
			
			if(status_Tag_seen==TRUE_CODE) // CR95HF woke up because of tag detection
			{
				CR95HF_Echo();// Check the communication
				LCD_Clear();// Refresh Display
				LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"  Tag seen ");
				// De init pin interruption 
				GPIO_Init(SPI_IRQ_IN_GPIO_PORT,SPI_IRQ_IN_GPIO_PIN ,GPIO_Mode_In_PU_No_IT);
				EXTI_DeInit();
				Wait_Display(WAIT_500ms);
				Select_Protocol_ISO15693();
			}
			else // refresh display
			{
				Wait_Display(WAIT_500ms);
			}
			
			// Note that the CR95HF can be turned into Sleep Mode (start state) or Hibernate mode ( very low consumption mode) but contactless cannot wakes up the CR95HF (only Interrupt). Please see the Application Note  Optimizing wakeup time and power consumption in CR95HF devices
			
			break;
			
			case JOYSTICK_SELSTATE:// sel
				// Add further action here
	
			break;
			
			default : break;
	}
	if(value==JOYSTICK_NO_BUTTON_PRESSED)
	{		
		delay_ms(WAIT_10ms);
	}
	

}
	
/**
* @brief This function returns the joystick state in order to detect an action on a button
* @param void : no parameter expected
* @retval void : no return value
*/
	static u8 GetJoystickState (void)
{
	// get the fifth first bits
	return ((GPIO_ReadInputData(JOYSTICK_GPIO_PORT)& JOYSTICK_NO_BUTTON_PRESSED)); 
}

/**  
* @brief  	This function initiate a tag detaction calibration for idle command
* @param void : no paramater expected
* @retval	 CR95HF_SUCCESS_CODE : command succeed
* @retval	 CR95HF_ERROR_CODE : command failed
*/
void Config_tag_detection(void)
{
	u8 status=CR95HF_ERROR_CODE;
	u8 time_counter=6;// .
	LED_Switch_on(ALL_LED);// activate  all LED
	LCD_PrintString(LCD_LINE1,ENABLE,DISABLE," TAG Detection");
	LCD_PrintString(LCD_LINE2,ENABLE,DISABLE,"  Calibration");
	Wait_Display(WAIT_500ms);
	LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"  Clear Field");
	status=CR95HF_Idle_Detector_Calibration(&Protocol_Settings, &Idle_Config);
	if(status==CR95HF_SUCCESS_CODE)
	{
		LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"Calibration OK");
		Wait_Display(WAIT_500ms);
		LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"##   CHECK   ##");
		LCD_PrintString(LCD_LINE2,ENABLE,DISABLE,"# Clear Field #");
		delay_ms(WAIT_1s);
		LCD_PrintString(LCD_LINE1,ENABLE,DISABLE," Test clr field");
		// Display 6 dots for 6 seconds
		LCD_PrintString(LCD_LINE2,ENABLE,DISABLE,"    ......  ");
		LCD_SetCursorPos(LCD_LINE2, (LCD_STARTLEFTCORNER+2));
		
		// Send idle with timer and tag detection to wake up.
		// the purpose of this command here is to check that the values found are correct: ie. no contactless is seen 
		CR95HF_Idle(	CR95HF_IDLE_WAKEUP_TIMER | CR95HF_IDLE_WAKEUP_TAG_DETECTION,
												CR95HF_DETECTOR_CALIBRATION_EnterCtrl_L,
												CR95HF_DETECTOR_CALIBRATION_EnterCtrl_H,
												CR95HF_DETECTOR_CALIBRATION_WUCtrl_L,
												CR95HF_DETECTOR_CALIBRATION_WUCtrl_H,
												CR95HF_IDLE_LEAVE_CTRL_L,
												CR95HF_IDLE_LEAVE_CTRL_H ,
												IDLE_TEST_WUP_160ms,
												CR95HF_IDLE_OSC_START,
												CR95HF_IDLE_DAC_START,
												Idle_Config.dacDataL,
												Idle_Config.dacDataH,
												CR95HF_IDLE_SWING_CNT,
												IDLE_TEST_MAXSLEEP_32_TRIALS,
												&Idle_Config);
	  															
		while(time_counter!=INIT_VALUE)
				{
					delay_ms(WAIT_1s);
					LCD_PrintChar(' ');// Erase each dot one by one
					time_counter--; // decrement remaining time
					/* Note that if a trial to resume the communication is made 
					before the CR95HF wakes up the procedure will fail.*/ 
				} 
		// Wait CR95HF wakes up and retrieve the cause
		status=CR95HF_Wait_Wake_up_from_Idle(&CR95HF_response,&Idle_Config,CR95HF_NUMBER_OF_WAITING_PERIOD);
		if(CR95HF_Is_Timeout_Wake_up_Condition(Idle_Config.last_reason_of_wakeup)==TRUE_CODE)
		{
			LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"Calibration OK");	
		}
				
		else
			LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"Calibration ERR");
	}
	else
		LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"Calibration ERR");
	
	LED_Switch_off(ALL_LED);// deactivate  all LED
	Wait_Display(WAIT_500ms);
}


/**
* @brief This function computes the protocol select parameter flags for iso15693 and sends a protocol select command to the CR95HF.
* @param void : no requested parameter
* @retval		CR95HF_SUCCESS_CODE : The command is successful 
* @retval		CR95HF_ERROR_INVALID_PROTOCOL : The protocol selected is wrong
* @retval		CR95HF_ERROR_INVALID_CMD_LENGTH : The command length is invalid  
* @retval 	CR95HF_ERROR_CODE unsuccessful command
*/
u8 Select_Protocol_ISO15693(void)
{
	// Compute the parameter byte, note other possibilities are commented
	ISO15693_ProtocolSelect_Parameters = CR95HF_ISO15693_ComputeParameterByte_ProtocolSelect
	(CR95HF_BIT7_6_RFU,
	CR95HF_BIT4_5_DATARATE_6KBITS,//CR95HF_BIT4_5_DATARATE_26KBITS, CR95HF_BIT4_5_DATARATE_52KBITS
	CR95HF_BIT3_WAIT_FOR_SOF,//CR95HF_BIT3_RESPECT312_DELAY
	CR95HF_BIT2_MODULATION_10,//CR95HF_BIT2_MODULATION_100
	CR95HF_BIT1_SINGLE_CARRIER);//CR95HF_BIT1_DUAL_CARRIERS
	
	return CR95HF_ProtocolSelect(CR95HF_PROT_SEL_ISO15693, CR95HF_PARAMETERSLENGTH_PROT_ISO15693, & ISO15693_ProtocolSelect_Parameters, &Protocol_Settings);
	 
}


/**  
* @brief  	This function holds the display for a fixed time and then clear the screen and turn it into the default message (logotype and name of the product).
* @param  	time:	time to wait before clear in ms
* @retval		void : no return value
*/
void Wait_Display(u16 Time)
{
	delay_ms(Time);// Wait
	LCD_Clear(); // Clear and display logo
	LCD_DisplayLogo(LCD_LOGOTYPE_LINE1_LEFT_CORNER );
	LCD_Print(" CR95HF  ");
}



