/**
  **********************************************************
  * @file cr95hf_command.h
  * @brief Header for cr95hf_command.c - constants for commands and parameters of communication
  * @author STMicroelectronics - MMY Application Team 
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
	
		
#ifndef __CR95HF_COMMAND_H
#define __CR95HF_COMMAND_H

// Includes ------------------------------------------------------------------
// Same level




//Lower level
#include "cr95hf_structure.h"
#include "cr95hf_driver.h"

// Generic
#include "string.h"


// Exported constants --------------------------------------------------------


// !< COMMANDS CODE
#define CR95HF_CMD_IDN 				0x01 
#define CR95HF_CMD_PROT_SEL		0x02 
#define CR95HF_CMD_SEND_REC 	0x04 
#define CR95HF_CMD_IDLE		 		0x07 
#define CR95HF_CMD_RDREG			0x08 
#define CR95HF_CMD_WRT_REG		0x09 
#define CR95HF_CMD_BAUDRATE 	0x0A 
#define CR95HF_CMD_ECHO     	0x55 

//---------------------------------------------------------------------------
// CUSTOM COMMANDS CODE  ST-RESERVED
#define CR95HF_CMD_POLL_FIELD		0x03 
#define CR95HF_CMD_LISTEN				0x05
#define CR95HF_CMD_SEND 				0x06 
#define CR95HF_CMD_SUB_FREQ_RES	0x0B 
#define CR95HF_CMD_AC_FILTER 		0x0D 
//----------------------------------------------------------------------------

// !< COMMANDS DATALENTH 
#define CR95HF_CMD_RD_REG_DATALENGTH									0x03 
#define CR95HF_CMD_BAUD_RATE_DATALENGTH								0x01 
#define CR95HF_CMD_IDLE_DATALENGTH										0x0E 
#define CR95HF_PARAMETERSLENGTH_FIELDOFF							0x01 
#define CR95HF_PARAMETERSLENGTH_PROT_ISO15693					0x01 
#define CR95HF_PARAMETERSLENGTH_PROT_ISO14443_BYTE0 	0x01  
#define CR95HF_PARAMETERSLENGTH_PROT_ISO14443_BYTE1 	0x02 
#define CR95HF_PARAMETERSLENGTH_PROT_ISO14443_BYTE2 	0x03 
#define CR95HF_PARAMETERSLENGTH_PROT_FELICA						0x02 

// PROTOCOLS
#define CR95HF_PROT_SEL_FIELDOFF 			0x00
#define CR95HF_PROT_SEL_ISO15693 			0x01
#define CR95HF_PROT_SEL_ISO14443_A 		0x02
#define CR95HF_PROT_SEL_ISO14443_B 		0x03
#define CR95HF_PROT_SEL_FELICA		 		0x04


// FIELD OFF PARAMETER BYTE
#define CR95HF_FIELDOFF_PARAMETER_BYTE 0x00


// PROTOCOL ISO15693 PARAMETERS 
#define CR95HF_BIT7_6_RFU 								0x00
#define CR95HF_BIT4_5_DATARATE_26KBITS		0x00
#define	CR95HF_BIT4_5_DATARATE_52KBITS		0x10
#define CR95HF_BIT4_5_DATARATE_6KBITS			0x20
#define CR95HF_BIT4_5_DATARATE_RFU				0x30
#define CR95HF_BIT3_RESPECT312_DELAY			0x00
#define CR95HF_BIT3_WAIT_FOR_SOF					0x08
#define CR95HF_BIT2_MODULATION_100				0x00
#define CR95HF_BIT2_MODULATION_10					0x04
#define CR95HF_BIT1_SINGLE_CARRIER				0x00
#define CR95HF_BIT1_DUAL_CARRIERS					0x02
#define CR95HF_BIT0_APPEND_CRC						0x01

#define CR95HF_DATARATE_MASK							0x30
#define CR95HF_SHIFT_DATARATE_INFO_TO_PARAMETERS 	0x04 // to shift the result of the get datarate command to a byte to use in the protocol select command

// CR95HF RESPONSE CODE
#define CR95HF_SUCCESS_CODE					0x00 //generic success code for command 
#define CR95HF_ERROR_CODE						0x01 //generic errorcode for command

// INIT VALUE
#define CR95HF_INIT_ResponseReceiving			0xEC // Error Check to prevent error from previous command 

// IDN command
#define CR95HF_IDN_SUCCESS_CODE						0x00
#define CR95HF_IDN_ROMCRC_CURSOR					0x02

// Protocol select command
#define CR95HF_PROTOCOL_SUCCESS_CODE			0x00
#define CR95HF_ERROR_INVALID_PROTOCOL			0x83
#define CR95HF_ERROR_INVALID_CMD_LENGTH		0x82

// SEND_RECV command
#define CR95HF_SENDRECV_SUCCESS_CODE				0x80
// most of the error described below could be used for other commands
#define CR95HF_ERROR_4BYTE_FRAME						0x90
#define CR95HF_ERROR_COMMUNICATION					0x86
#define CR95HF_ERROR_NOTAG_OR_TIMEOUT				0x87
#define CR95HF_ERROR_INVALID_SOF						0x88
#define CR95HF_ERROR_BUFFER_OVERFLOW				0x89
#define CR95HF_ERROR_FRAMING								0x8A
#define CR95HF_EROR_EGT_TIMEOUT							0x8B
#define CR95HF_ERROR_INVALID_LENGTH_FELICA	0x8C
#define CR95HF_ERROR_CRC_FELICA							0x8D
#define CR95HF_ERROR_RECEPTION_LOST					0x8E

// Read Register commands
#define CR95HF_RD_REG_SUCCESS_CODE						0x00
#define CR95HF_RD_REG_ST_RESERVED							0x00
// wake up register
#define CR95HF_WAKE_UP_REGISTER_CURSOR				0x62
#define CR95HF_WAKE_UP_REGISTER_SIZE					0x01
// Analog configuration register
#define CR95HF_ANALOG_CONFIG_REGISTER_CURSOR	0x69
#define CR95HF_ANALOG_CONFIG_REGISTER_SIZE		0x01

// Write register commands
#define CR95HF_ANALOG_CONFIG_REGISTER_INDEX		0x68
#define CR95HF_ANALOG_CONFIG_INDEX						0x01

//  Write RCB Register
#define CR95HF_WR_REG_LENGTH_OF_CMD						0x04
#define CR95HF_WR_REG_FLAG_INCREMENT					0x01

//  Set RCB Index command
#define CR95HF_SET_INDEX_LENGTH_OF_CMD				0x03
#define CR95HF_SET_INDEX_FLAG_INCREMENT				0x00

// To compare with register value in order to determine what leaded to exit the last idle mode
#define CR95HF_WAKE_UP_BECAUSE_TAG_DETECTION	0x02 
#define CR95HF_WAKE_UP_BECAUSE_TIMEOUT				0x01

// Baud rate command 
#define CR95HF_DEFAULT_BAUD_RATE									0x75
#define CR95HF_BAUD_RATE_FAILED_DEFAULT_VALUE_ON	0x10
#define CR95HF_BAUD_RATE_FAILED_PREVIOUS_VALUE_ON	0x20

// ECHO command
#define CR95HF_ECHO_SUCCESS_CODE					0x55

// Baud rate error
#define CR95HF_BAUD_RATE_ERROR		0xBE

// TimeOut error
#define CR95HF_COMMUNICATION_WITH_MCU_FAILED	0xCE // Communication Error 

// Protocol parameters response for Get request
// Invalid protocol
#define CR95HF_INVALID_PROTOCOL			0xE0
#define CR95HF_PARAMETER_BYTE0			0x00
#define CR95HF_PARAMETER_BYTE1			0xE0
#define CR95HF_PARAMETER_BYTE2			0xE0
// ISO15693 parameters response for getting info functions
#define CR95HF_DATARATE_26KBITS			0x00
#define	CR95HF_DATARATE_52KBITS			0x01 
#define CR95HF_DATARATE_6KBITS			0x02 
#define CR95HF_DATARATE_RFU					0x03 
#define CR95HF_RESPECT312_DELAY			0x00
#define CR95HF_WAIT_FOR_SOF					0x01
#define CR95HF_MODULATION_100				0x00
#define CR95HF_MODULATION_10				0x01 
#define CR95HF_SINGLE_CARRIER				0x00
#define CR95HF_DUAL_CARRIERS				0x01 


// Idle command 
#define CR95HF_CMD_IDLE_SUCCESS_CODE 			0x00
//defaults parameters and flags paramaters
#define CR95HF_IDLE_LEAVE_CTRL_L 					0x18
#define CR95HF_IDLE_LEAVE_CTRL_H 					0x00
#define CR95HF_IDLE_OSC_START   					0x60
#define CR95HF_IDLE_DAC_START	  					0x60
#define CR95HF_IDLE_SWING_CNT	  					0x3F

// Bits for Ctrl_L and Ctrl_H
/*
CtrlL	
	 7  Initial DAC compare index
   6  DAC enable    			5  LFO enable
   4  HFO enable    			3  VddA enable
   2  Hibernate enable  	1  Idle mode enable
   0  Sleep mode enable

CtrlH	
   7  RFU  6  GtmStop (when set, can save ~3uA in Sleep (WU by SS will be disabled))
   5  CLK source (0 = HFO; 1 = CLK_REC) Must be activated independently!
   4  Card clamp enable				3  Card RX chain enable
   2  Reader RX chain enable 	1  FDet enable
   0  IRef enable
*/
#define CR95HF_IDLE_CRTRL_L_INIT_DAC_CMP_IDX 	0x80
#define CR95HF_IDLE_CRTRL_L_DAC_ENABLE				0x40	
#define CR95HF_IDLE_CRTRL_L_LFO_ENABLE				0x20
#define CR95HF_IDLE_CRTRL_L_HFO_ENABLE				0x10
#define CR95HF_IDLE_CRTRL_L_VDDA_ENABLE				0x08
#define CR95HF_IDLE_CRTRL_L_HIBERNATE_ENABLE	0x04
#define CR95HF_IDLE_CRTRL_L_IDLE_MODE_ENABLE	0x02
#define CR95HF_IDLE_CRTRL_L_SLEEP_MODE_ENABLE	0x01

#define CR95HF_IDLE_CRTRL_H_GTM_STOP								0x40
#define CR95HF_IDLE_CRTRL_H_CLK_SRC_HFO							0x00
#define CR95HF_IDLE_CRTRL_H_CLK_SRC_CLK_REC					0x02
#define CR95HF_IDLE_CRTRL_H_CARD_CLAMP_ENABLE				0x01	
#define CR95HF_IDLE_CRTRL_H_CARD_RX_CHAIN_ENABLE		0x08
#define CR95HF_IDLE_CRTRL_H_READER_RX_CHAIN_ENABLE	0x04
#define CR95HF_IDLE_CRTRL_H_F_DET_ENABLE						0x02
#define CR95HF_IDLE_CRTRL_H_IRef_ENABLE							0x01


#define CR95HF_IDLE_MASK_WAKE_UP_FLAGS 				0x0F

#define CR95HF_IDLE_WUFLAGS_BIT0_TIMER 0x01 
/* CR95HF wakes up  and returns in ready state at the end of  a predefine cycle Time Out is defined by Max sleep and Wake Up period  , 
TO = (Maxsleep *( WuPeriod+1)*RT ,     RT= 256*TL = 8ms
This bit must be set when using theTimer as a possible Wake-Up source .
 It must be set during TAG calibration to force Wake Up after the first Tag detection trial.*/
#define CR95HF_IDLE_WUFLAGS_BIT1_TAG_DETECTED 	0x02 
// Wake up when a Tag is detected ,  must also be set during Tag detector Calibration or Tag detection .
#define CR95HF_IDLE_WUFLAGS_BIT2_RFU 0x00
#define CR95HF_IDLE_WUFLAGS_BIT3_EXTERNAL_INTERRUPT_IRQ_IN 0x08
/*Wake Up by an external Interrupt , low level on |IRQ_IN (IO7) , 
useful in SPI It is recommended to set this bit in case of system crash .*/
#define CR95HF_IDLE_WUFLAGS_BIT4_EXTERNAL_INTERRUPT_NSS		 0x10
// Wake Up by an external Interrupt , low level on |SS (IO57) , Useful in UART
#define CR95HF_IDLE_WUFLAGS_BIT5_RFU 0x00
#define CR95HF_IDLE_WUFLAGS_BIT6_RFU 0x00
#define CR95HF_IDLE_WUFLAGS_BIT7_RFU 0x00

#define CR95HF_IDLE_WUFLAGS_BIT_RESET 0x00

// Detector calibration parameters value
#define CR95HF_DETECTOR_CALIBRATION_EnterCtrl_L 	0xA2
#define CR95HF_DETECTOR_CALIBRATION_EnterCtrl_H 	0x00
#define CR95HF_DETECTOR_CALIBRATION_WUCtrl_L 			0xF8
#define CR95HF_DETECTOR_CALIBRATION_WUCtrl_H 			0x01
#define CR95HF_DETECTOR_CALIBRATION_LeaveCtrl_L 	0x18
#define CR95HF_DETECTOR_CALIBRATION_LeaveCtrl_H 	0x00
#define CR95HF_DETECTOR_CALIBRATION_WUPeriod			0x01
#define CR95HF_DETECTOR_CALIBRATION_Max_Sleep			0x01
#define CR95HF_DETECTOR_CALIBRATION_REF_0         0x3F
#define CR95HF_DETECTOR_CALIBRATION_LOOP_CNT_INIT 0x01
#define CR95HF_DETECTOR_CALIBRATION_LOOP_CNT_END	0x06
#define CR95HF_DETECTOR_CALIBRATION_DACDATA_H_MIN	0x00
#define CR95HF_DETECTOR_CALIBRATION_DACDATA_H_MAX	0x3F
#define CR95HF_DETECTOR_CALIBRATION_DACDATA_L			0x00
#define CR95HF_NUMBER_OF_WAITING_PERIOD						0x01// for tag calibration procedure


//--------------------------------------------------------------------
// CUSTOM CONSTANTS CODE ST-RESERVED

// ERROR
#define CR95HF_ERROR		0xEE // To detect error for comands which do not return error (for instance : poll field, baud rate)
#define CR95HF_POLL_FIELD_CMD_FLAG_SET   			0x01
#define	CR95HF_POLL_FIELD_CMD_FLAG_RESET			0x00
#define	CR95HF_POLL_FIELD_CMD_SUCCESS_CODE		0x00
#define	CR95HF_POLL_FIELD_CMD_DATALENGTH			0x01

#define CR95HF_LISTEN_CMD_SUCCESS_CODE				0x00
#define CR95HF_LISTEN_CMD_COMMUNICATION_SUCCESS_CODE		0x80
#define CR95HF_LISTEN_ERROR_NOFIELD						0x8F

#define CR95HF_SEND_CMD_SUCCESS_CODE					0x00
#define CR95HF_SEND_CMD_RESPONSE_DATALENGTH		0x00

#define CR95HF_WRITE_REG_CMD_SUCCESS_CODE			0x00
#define CR95HF_SUB_FREQ_RES_CMD_SUCCESS_CODE	0x00
#define CR95HF_SUB_FREQ_RES_CMD_RESPONSE_DATALENGTH		0x01

#define	 CR95HF_ACFILTER_CMD_SUCCESS_CODE	0x00
#define  CR95HF_ACFILTER_ATQA_LENGTH			0x02
#define  CR95HF_ACFILTER_UID_PART_LENGTH 	0x04
#define  CR95HF_ACFILTER_LENGTH_7					0x07
#define  CR95HF_ACFILTER_LENGTH_11				0x0B

//--------------------------------------------------------------

//--------------------------------------------------------------------
//functions ----------------------------------------------------------  


// Command functions

u8 CR95HF_IDN (CR95HF_ResponseReceiving* Response);

u8 CR95HF_ProtocolSelect (const u8 Protocol, const u8 ParametersLength,const u8* Parameters, CR95HF_Protocol_Config* Protocol_Settings);

u8 CR95HF_SendRecv(CR95HF_ResponseReceiving* Response,const  CR95HF_CommandSending* Command);

u8 CR95HF_Idle (const  u8 WakeUpFlags,const  u8 EnterCtrlL,const  u8 EnterCtrlH, const u8 WUCtrlL,const  u8 WUCtrlH,const  u8 LeaveCtrlL,const  u8  LeaveCtrlH ,const u8 WUPeriod,const  u8 OscStart,const  u8 DacStart,const  u8 DacDataL,const  u8 DacDataH,const  u8 SwingsCnt,const  u8 MaxSleep, CR95HF_Idle_Config* Idle_Config);

u8 CR95HF_Rd_Wake_up_Reg(CR95HF_ResponseReceiving* Response,CR95HF_Idle_Config* Idle_Config);

u8 CR95HF_Rd_Analog_Register_Config_B(CR95HF_ResponseReceiving* Response);

u8 CR95HF_Set_Analog_Register_Config_B_Index( void);

u8 CR95HF_Wr_Analog_Register_Config_B_Reg (u8 Analog_Configuration_Register);

u8 CR95HF_Baud_Rate(const u8 New_Baud_Rate);

u8 CR95HF_Echo (void);

u8 CR95HF_FieldOff(CR95HF_Protocol_Config* Protocol_Settings);

u8 CR95HF_SendEOF(CR95HF_ResponseReceiving* Response);

u8 CR95HF_Modify_Baud_Rate(const u8 New_Baud_Rate);

// Support functions 

u8 CR95HF_IsReaderResultOK(CR95HF_ResponseReceiving* Response,const u8 Code_OK);

u8 CR95HF_ISO15693_ComputeParameterByte_ProtocolSelect
( u8 Bit7_6,u8 Bit5_4,u8 Bit3,u8 Bit2,u8 Bit1);

static void CR95HF_Update_Protocol_Selected( CR95HF_Protocol_Config* Protocol_Settings, u8 Status,
u8 Protocol,u8 parameters_Length, u8* Parameters);

u8 CR95HF_Is_IRQin_Interrupt_Wake_up_Condition(const u8 WUFlags);

u8 CR95HF_Is_NSS_Interrupt_Wake_up_Condition(const u8 WUFlags);

u8 CR95HF_Is_Tag_Detected_Wake_up_Condition(const u8 WUFlags);

u8 CR95HF_Is_Timeout_Wake_up_Condition(const u8 WUFlags);

u8 CR95HF_Idle_Detector_Calibration(CR95HF_Protocol_Config* Protocol_Settings,CR95HF_Idle_Config* Idle_Config);

static u8 CR95HF_Tagcal(const u8 DacDataL, const u8 DacDataH);

u8 CR95HF_Tagdet(const u8 DacDataL, const u8 DacDataH);

u8 CR95HF_Wait_Wake_up_from_Idle(CR95HF_ResponseReceiving* CR95HF_Response_Idle,CR95HF_Idle_Config* Idle_Config, u8 Number_of_DELAY_BEFORE_TIMEOUT_250MS);

u8 CR95HF_Wake_up_CR95HF(CR95HF_Idle_Config* Idle_Config);


// private functions

void CR95HF_SendCommand(CR95HF_CommandSending* Cmd);

CR95HF_ResponseReceiving CR95HF_ReceiveAnswer(void);

static CR95HF_ResponseReceiving CR95HF_SendReceive_Cmd(CR95HF_CommandSending* Cmd);

void  CR95HF_Wait_Receive_Answer(CR95HF_ResponseReceiving* Response);

void CR95HF_Init_ResponseReceiving(CR95HF_ResponseReceiving* Response);

void CR95HF_Add_Data_ToCommand(CR95HF_CommandSending* Cmd, u8* Cptr_ToIncrement, u8  DataLength, u8* Data);

// Custom Commands ST-Reserved

#ifdef ST_RESERVED_FUNCTIONS_AVAILABLE
CR95HF_ResponseReceiving CR95HF_Poll_Field(void);

u8 CR95HF_Listen(CR95HF_ResponseReceiving* Response);

u8 CR95HF_Send( CR95HF_CommandSending* Command);

u8 CR95HF_Rd_Reg( CR95HF_ResponseReceiving* Response, u8 Address, u8 RegCount, u8  Flags);

u8 CR95HF_Wr_Reg( u8* Reg_Data, u8 Address, u8 RegCount, u8  Flags);

u8 CR95HF_SubFreqRes(u8* Sub_freq_divider);

u8 CR95HF_AcFilter(u8 Length, u8* ATQA, u8 SAK, u8* UID_part_1, u8* UID_part_2, u8* UID_part_3 );
#endif 

#endif /*__CR95HF_COMMAND_H */
	
