/**
  ******************************************************************************
  * @file    CR95HF_STM8L_Header.c
  * @author  MMY Application Team
  * @version V1.0
  * @date    19/09/11
  * @brief   This file provides a set of function to manage the CR95HF, and communicate with ISO15693 contactless tags.
  ******************************************************************************

/**
 * <!-- ------------------------------------------------------------------------------------------------------------ -->
 * @mainpage
 * @section libSTM8L_CR95HFMainIntroduction Introduction
 * This document describes the CR95HF library and the libraries dedicated to communication with contactless tags compliant with the ISO/IEC 15693 specification. 
 *
 * This firmware provides a set functions to manage the CR95HF, and communicate with ISO15693 contactless tags and moreover with LRI*K, M24LR* families.
 * <!-- ------------------------------------------------------------------------------------------------------------ -->
 * @subsection XorroMainTerms General Description
 * The librairies are organized into three layers: 
 * 
 * - The fist layer contains a set of commands and functions to manage the CR95HF 
 *
 * - The second layer contains the set of commands and functions to communicate with ISO15693 compliant contactless tags using the CR95HF
 *
 * - The third layer includes the set of commands and functions dedicated to STMicroelectronics family products compliant with ISO1593 protocol.
 *
 * The graphic below represents the organization of the layers into a application. 
 * 
 * <TABLE>
 *	<TR>
 *		<TD>_________   User Application   _________</TD>
 *		
 *  </TR>   
 * </TABLE>
 * <TABLE>
 *	<TR>
 *		<TD>____<b>   M24LR*  </b>____ </TD>
 *
 *		<TD> ____<b>    LRI*K   </b> ____</TD>
 *
 *  </TR>  
 * </TABLE>
 * <TABLE>
 *	<TR>
 *		<TD>_______  <b> ISO/IEC 15693 Layer</b>  _____</TD>
 *
 *  </TR>
 *	<TR>
 *		<TD>_________   <b> CR95HF Layer</b>  _________</TD>
 *		
 *  </TR>   
 *	<TR>
 *		<TD>___________  Hardware  ____________</TD>
 *		
 *  </TR>   
 * </TABLE>
 *
 * <b> Terms and conventions</b>  
 *
 * The table provides a list of terms used in this document and their meaning.
 *
 * <TABLE>
 *	<TR>
 *		<TD><b>Term</b></TD>
 *		<TD><b>Description</b></TD>
 *  </TR>  
 *	<TR>
 *		<TD>AFI</TD>
 *		<TD>Application Family Identifier</TD>
 *  </TR>  
*	<TR>
 *		<TD>CRC</TD>
 *		<TD>Cyclic Redundancy Check</TD>
 *  </TR>   
  *	<TR>
 *		<TD>DSFID</TD>
 *		<TD>Data Storage Format IDentifier </TD>
 *  </TR>
 *	<TR>
 *		<TD>ICRef</TD>
 *		<TD>Integrated Circuit Reference </TD>
 *  </TR>
  *	<TR>
 *		<TD>LRI*k</TD>
 *		<TD>Long Range Interface </TD>
 *  </TR>
 *	<TR>
 *		<TD>M24LR*</TD>
 *		<TD>Dual interface EEPROM (I2C and RF) with 64kbits memory size </TD>
 *  </TR>
 *	<TR>
 *		<TD>M24LR*-E</TD>
 *		<TD>Dual interface EEPROM (I2C and RF) with 64kbits memory size & Energy harvesting feature </TD>
 *  </TR>
 *	<TR>
 *		<TD>MCU</TD>
 *		<TD>Microcontroller Unit </TD>
 *  </TR>
 *	<TR>
 *		<TD>RFID</TD>
 *		<TD>Radio frequency Idedentification</TD>
 *  </TR>  
 *	<TR>
 *		<TD>RFU</TD>
 *		<TD>Reserved for Future Use</TD>
 *  </TR>  
  *	<TR>
 *		<TD>UID</TD>
 *		<TD>Unique IDentifier </TD>
 *  </TR>  
 * </TABLE>
 *
 * <b> Disclaimer</b>  
 *
* The present firmware which is for guidance only aims at providing customers with coding informations regarding their products in order for them to save time.
*
* As a result STMicroelectronics shalll not be held liable for any direct indirect or consequential damages with respect to any claims arising from the content of such firmware and/or the use made by customers of the coding  infomation contained herein in connection with their products.
*
 *
 * @page STM8L_CR95HFGroupChangeLog Revision History
 * <TABLE>
 *		<CAPTION>Document revision history</CAPTION>
 *		<TR><TD><B>Date</B></TD><TD><B>Revision</B></TD><TD><B>Changes</B></TD></TR>
 *		<TR><TD>01-October-2011</TD><TD>1.0</TD><TD>Initial release</TD></TR>
 * </TABLE>
*******************************************************************************/


