/**
  **********************************************************
  * @file lrixk_command.h
  * @brief Header for the lrixk_command.c
  * @author STMicroelectronics - MMY Application Team 
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
#ifndef __LRIXK_COMMAND_H
#define __LRIXK_COMMAND_H	
	
	
// Includes ----------------------------------------------
#include "iso15693_command.h"

// Custom commands code
#define LRIXK_CMD_KILL														0xA6
#define LRIXK_CMD_WRITE_KILL											0xB1
#define LRIXK_CMD_LOCK_KILL												0xB2
#define LRIXK_CMD_LOCK_KILL												0xB2
#define LRIXK_CMD_FAST_READ_SINGLE_BLOCK					0xC0
#define LRIXK_CMD_FAST_INVENTORY_INITIATED				0xC1
#define LRIXK_CMD_FAST_INITIATE										0xC2
#define LRIXK_CMD_FAST_READ_MULTIPLE_BLOCKS				0xC3
#define LRIXK_CMD_INVENTORY_INITIATED							0xD1
#define LRIXK_CMD_INITIATE												0xD2

// Constants for returning value
#define LRIXK_SUCCESS_CODE										0x00
#define LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS	0x02
#define LRIXK_ERROR_CODE											0x01
#define LRIXK_BLOCK_SIZE_ERROR								0x00
#define LRIXK_ERROR_CRC  										0xCC // arbritrary defined constant

// others constants for commands specific to LRIXK family
#define LRIXK_DEFAULT_BLOCK_SIZE							0x04
#define LRIXK_KILL_ACCESS											0x00
#define LRIXK_KILL_PROTECT_STATUS							0x01

// For Request flags
// !! For all commands Protocol extension flag is set to 0 and is included in all the constants
#define LRIXK_MASK_REQUEST_NO_PROT_EXTENSION								0x08	
#define LRIXK_BIT8_LOCK_KILL_RFU_FLAG												0x80	
#define LRIXK_MASK_REQUEST_FLAGS_INITIATE										0x7C	
#define LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY_OR_OPTION			0x4C	
#define LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY_OPTION_SELECT	0x5C	
#define LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY								0x0C	
#define LRIXK_MASK_REQUEST_FLAGS_NO_OPTION									0x48	

	
//------------------------------------------------------------------------

u8 LRIXK_Inventory(ISO15693_Tag* MyTag, u8 Request_flags, const u8 Tag_AFI, 
const u8 MaskLength,const u8* Mask,  u8* Inventory_16_slots_Nb_UID_Retrieved);

u8 LRIXK_Stay_Quiet( const u8 Request_flags, const u8* Tag_UID);

u8 LRIXK_Read_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Block_number);

u8 LRIXK_Write_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Block_number, const u8* Data);

u8 LRIXK_Lock_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Block_number);

u8 LRIXK_Read_Multiple_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 First_Block_number, const u8 Number_of_Blocks);

u8 LRIXK_Select(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID);

u8 LRIXK_Reset_to_Ready(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID);

u8 LRIXK_Write_AFI(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Tag_AFI);

u8 LRIXK_Lock_AFI(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID);

u8 LRIXK_Write_DSFID(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Tag_DSFID);

u8 LRIXK_Lock_DSFID(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID);

u8 LRIXK_Get_System_Info(ISO15693_Tag* MyTag,const   u8 Request_flags,const u8* Tag_UID);

u8 LRIXK_Get_Multiple_Blocks_Security_Status(ISO15693_Tag* MyTag,  u8 Request_flags, const u8* Tag_UID, const u8 First_Block_number, const u8 Number_of_Blocks);

// Custom code command -------------------------------------------------------------

u8 LRIXK_Kill(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8* Kill_Code);

u8 LRIXK_Write_Kill(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8* Kill_Code);

u8 LRIXK_Lock_Kill(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID);

u8 LRIXK_Fast_Read_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Block_number);

u8 LRIXK_Fast_Inventory_Initiated(ISO15693_Tag* MyTag, const u8 Request_flags, const u8 Tag_AFI, const u8 MaskLength, const u8* Mask, u8* Inventory_16_slots_Nb_UID_Retrieved);

u8 LRIXK_Fast_Initiate(ISO15693_Tag* MyTag, const u8 Request_flags);

u8 LRIXK_Fast_Read_Multiple_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 First_Block_number, const u8 Number_of_Blocks);

u8 LRIXK_Inventory_Initiated(ISO15693_Tag* MyTag, const u8 Request_flags, const u8 Tag_AFI, const u8 MaskLength, const u8* Mask, u8* Inventory_16_slots_Nb_UID_Retrieved);

u8 LRIXK_Initiate(ISO15693_Tag* MyTag, const u8 Request_flags);


// support functions ------------------------------------------------------------------

u8 LRIXK_Is_Request_Flags_ok(const u8 Request_flags, const u8 ISO15693_LRIXK_cmd_code);

u8 LRIXK_Is_Memory_programming_function(const u8 ISO15693_LRIXK_cmd_code);

u8 LRIXK_Is_Fast_Datarate_ok(const u8 Request_flags,const CR95HF_Protocol_Config* Protocol_Settings);

u8 LRIXK_Is_Fast_Command(const u8 LRIXK_cmd_code);

#endif /*__LRIXK_COMMAND_H */
