/**
  **********************************************************
  * @file m24lrxx_command.h
  * @brief Header for the m24lrxx_command.c
  * @author STMicroelectronics - MMY Application Team 
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
	
	
#ifndef __M24LRXX_COMMAND_H
#define __M24LRXX_COMMAND_H

// Includes ----------------------------------------------
#include "iso15693_command.h"


// Uncomment the line below to access all the specific commands linked to energy harvesting products (M14LRXX-E)
#define USE_ENERGY_HARVESTING_COMMANDS


// Custom commands code
#define M24LRXX_CMD_GET_MULTIPLE_BLOCK_SECURITY_STATUS	0x2C
#define M24LRXX_CMD_WRITE_SECTOR_PASSWORD								0xB1
#define M24LRXX_CMD_LOCK_SECTOR_PASSWORD								0xB2
#define M24LRXX_CMD_PRESENT_SECTOR_PASSWORD							0xB3
#define M24LRXX_CMD_FAST_READ_SINGLE_BLOCK							0xC0
#define M24LRXX_CMD_FAST_INVENTORY_INITIATED						0xC1
#define M24LRXX_CMD_FAST_INITIATE												0xC2
#define M24LRXX_CMD_FAST_READ_MULTIPLE_BLOCKS						0xC3
#define M24LRXX_CMD_INVENTORY_INITIATED									0xD1
#define M24LRXX_CMD_INITIATE														0xD2
// specific to energy harvesting devices
#define M24LRXX_CMD_READ_CFG														0xA0 
#define M24LRXX_CMD_WRITE_EH_CFG												0xA1
#define M24LRXX_CMD_SET_RST_EH_EN												0xA2
#define M24LRXX_CMD_CHECK_EH_EN													0xA3
#define M24LRXX_CMD_WRITE_DO_CFG												0xA4

// Constants for returning value
#define M24LRXX_SUCCESS_CODE										0x00
#define M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS	0x02
#define M24LRXX_ERROR_CODE											0x01
#define M24LRXX_BLOCK_SIZE_ERROR								0x00
#define M24LRXX_ERROR_CRC  											0xCC // arbritrary defined constant




// Command error Code specific to M24LRXX family
#define M24LRXX_SPECIFIED_BLOCK_READ_PROTECTED		0x15


// others constants for commands specific to M24LRXX family
#define M24LRXX_DEFAULT_BLOCK_SIZE							0x04
#define M24LRXX_DEFAULT_SECTOR_SIZE							0x20 // in blocks
// Mask for request flags
// in order to force protocol extension flag and option flag to 0
#define M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_EXTENSION_FLAG 									0x48	
#define M24LRXX_MASK_REQUEST_FLAGS_NO_EXTENSION_INVENTORY_FLAG 								0x0C	
#define M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_INVENTORY_FLAG 									0x44	
#define M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_EXTENSION_INVENTORY_FLAG 				0x4C	
#define M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_EXTENSION_INVENTORY_SELECT_FLAG 	0x5C	
#define M24LRXX_MASK_REQUEST_FLAGS_NO_EXTENSION_FLAG 													0x08	
#define M24LRXX_MASK_REQUEST_FLAGS_CMD_INITIATE																0x7C 



// Constant for Security Status  ---------------------------------------

#define M24LRXX_SECURITY_STATUS_BLOCK_LOCKED				   			 0x01 

// Constants for PASSWORD
#define M24LRXX_SECURITY_STATUS_MASK_PASSWORD_CTRL				   0x18
// Password control bits
#define M24LRXX_SECURITY_STATUS_NO_PASSWORD				   				 0x00 
#define M24LRXX_SECURITY_STATUS_PASSWORD_1				   		 		 0x08 
#define M24LRXX_SECURITY_STATUS_PASSWORD_2				   		 		 0x10 
#define M24LRXX_SECURITY_STATUS_PASSWORD_3				   		 		 0x18

// constants for read write protection
#define M24LRXX_SECURITY_STATUS_MASK_READ_WRITE_PROTECTION   0x06 
// presenting password
#define M24LRXX_SECURITY_STATUS_WITH_PASSWORD_READ_WRITE		 0x00 
// others configuration 01 10 are also read write  
#define M24LRXX_SECURITY_STATUS_WITH_PASSWORD_READ_ONLY			 0x06
// without password
#define M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_READ_ONLY	 			 0x00
#define M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_READ_WRITE	 			 0x02
#define M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_NO_READ_NO_WRITE	 0x04
// 11 is also a no read no write configuration

// For energy harvesting --------------------------------------------------
// configuration byte
#define M24LRXX_CONFIGURATION_BYTE_CURSOR 					0x01 // into the tag structure

// Range
#define M24LRXX_CONFIGURATION_BYTE_MASK_RANGE 			0x03 
#define M24LRXX_CONFIGURATION_BYTE_RANGE_6_MA		 		0x00
#define M24LRXX_CONFIGURATION_BYTE_RANGE_3_MA				0x01
#define M24LRXX_CONFIGURATION_BYTE_RANGE_1_MA 			0x02
#define M24LRXX_CONFIGURATION_BYTE_RANGE_300_UA 		0x03

//RF busy Write in progress
#define M24LRXX_CONFIGURATION_BYTE_MASK_RF_WIP_BUSY 0x08
#define M24LRXX_CONFIGURATION_BYTE_RF_BUSY				  0x00
#define M24LRXX_CONFIGURATION_BYTE_RF_WIP					  0x08

//EH mode
#define M24LRXX_CONFIGURATION_BYTE_MASK_EH_MODE		  0x04
#define M24LRXX_CONFIGURATION_BYTE_EH_ENABLED  			0x00
#define M24LRXX_CONFIGURATION_BYTE_EH_DISABLED 			0x04

// Control Register
#define M24LRXX_CONTROL_REGISTER_CURSOR 				0x01 // into the tag structure
#define M24LRXX_CONTROL_REGISTER_EH_ENABLE 			0x01
#define M24LRXX_CONTROL_REGISTER_EH_DISABLE 		0x00
//------------------------------------------------------------------------


u8 M24LRXX_Inventory(ISO15693_Tag* MyTag, const u8 Request_flags, const u8 Tag_AFI, 
const u8 MaskLength,const u8* Mask,  u8* Inventory_16_slots_Nb_UID_Retrieved);

u8 M24LRXX_Stay_Quiet(const u8 Request_flags,const  u8* Tag_UID);

u8 M24LRXX_Read_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u16 Block_number);

u8 M24LRXX_Write_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u16 Block_number, const u8* Data);

u8 M24LRXX_Read_Multiple_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u16 First_Block_number, const u8 Number_of_Blocks);

u8 M24LRXX_Select(ISO15693_Tag* MyTag,const u8 Request_flags, const u8* Tag_UID);

u8 M24LRXX_Reset_to_Ready(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID);

u8 M24LRXX_Write_AFI(ISO15693_Tag* MyTag,  const u8 Request_flags,const  u8* Tag_UID,const  u8 Tag_AFI);

u8 M24LRXX_Lock_AFI(ISO15693_Tag* MyTag,  const u8 Request_flags,const  u8* Tag_UID);

u8 M24LRXX_Write_DSFID(ISO15693_Tag* MyTag,  const u8 Request_flags,const  u8* Tag_UID,const  u8 Tag_DSFID);

u8 M24LRXX_Lock_DSFID(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID);

u8 M24LRXX_Get_System_Info(ISO15693_Tag* MyTag,  const u8 Request_flags,const u8* Tag_UID);


// Custom code command -------------------------------------------------------------

u8 M24LRXX_Get_Multiple_Blocks_Security_Status(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u16 First_Block_number, const u8 Number_of_Blocks);

u8 M24LRXX_Write_Sector_Password(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID,const u8 Password_Number ,const u8* Password);

u8 M24LRXX_Lock_Sector_Password(ISO15693_Tag* MyTag, u8 Request_flags, const u8* Tag_UID, u16 Sector_Number, u8 Sector_Security_Status);

u8 M24LRXX_Present_Sector_Password(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID,const u8 Password_Number ,const u8* Password);

u8 M24LRXX_Fast_Read_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u16 Block_number);

u8 M24LRXX_Fast_Inventory_Initiated(ISO15693_Tag* MyTag,const  u8 Request_flags, const u8 Tag_AFI, const u8 MaskLength, const u8* Mask, u8* Inventory_16_slots_Nb_UID_Retrieved);

u8 M24LRXX_Fast_Initiate(ISO15693_Tag* MyTag, const u8 Request_flags);

u8 M24LRXX_Fast_Read_Multiple_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 First_Block_number, const u8 Number_of_Blocks);

u8 M24LRXX_Inventory_Initiated(ISO15693_Tag* MyTag, const u8 Request_flags, const u8 Tag_AFI, const u8 MaskLength, const u8* Mask, u8* Inventory_16_slots_Nb_UID_Retrieved);

u8 M24LRXX_Initiate(ISO15693_Tag* MyTag, const u8 Request_flags);


// Energy harvesting commands


#ifdef USE_ENERGY_HARVESTING_COMMANDS

u8 M24LRXX_ReadCfg(ISO15693_Tag* MyTag,  const u8 Request_flags,const u8* Tag_UID) ;

u8 M24LRXX_Write_EH_Cfg(ISO15693_Tag* MyTag,  const u8 Request_flags,const u8* Tag_UID, u8 Configuration_Byte) ;

u8 M24LRXX_Write_DO_Cfg(ISO15693_Tag* MyTag,  const u8 Request_flags,const u8* Tag_UID, u8 Configuration_Byte) ;

u8 M24LRXX_SetRst_EH_en(ISO15693_Tag* MyTag,  const u8 Request_flags,const u8* Tag_UID, u8 Set_Reset) ;

u8 M24LRXX_Check_EH_En(ISO15693_Tag* MyTag,  const u8 Request_flags,const u8* Tag_UID) ;

#endif

// -----------------------------------------------------------------------------
// Support functions -----------------------------------------------------------

// Is test functions -----------------------------------------------------------

u8 M24LRXX_Is_Request_Flags_ok(const u8 Request_flags, const u8 M24LRXX_cmd_code);

u8 M24LRXX_Is_Memory_programming_function(const u8 M24LRXX_cmd_code);

u8 M24LRXX_Is_Fast_Datarate_ok(const u8 Request_flags,const CR95HF_Protocol_Config* Protocol_Settings);

u8 M24LRXX_Is_Fast_Command(const u8 M24LRXX_cmd_code);

// Get parameters functions ----------------------------------------------------

u8 M24LRXX_Get_Block_Security_Password_number(u8 Byte_Status);

u8 M24LRXX_Get_Block_Security_Read_Write_Protection_with_Password(u8 Byte_Status);

// Energy harvesting -----------------------------------------------------

#ifdef USE_ENERGY_HARVESTING_COMMANDS
u8 M24LRXX_Get_Energy_Harvesting_Range( u8 Configuration_Byte);

u8 M24LRXX_Get_RF_BUSY_WIP(u8 Configuration_Byte);

u8 M24LRXX_Get_EH_mode_Configuration_Byte(u8 Configuration_Byte);

u8 M24LRXX_Get_EH_mode_Control_Register(u8 Control_Register);
#endif

//------------------------------------------------------------------------------

u8 M24LRXX_Compute_Security_Status_byte(u8 Password_Number, u8 Read_Write_Protection_Password, u8 Read_Write_Protection_No_Password);

#ifdef USE_ENERGY_HARVESTING_COMMANDS
	u8 M24LRXX_Compute_Configuration_Byte(u8 RF_WIP_BUSY,u8 EH_Enable,u8 EH_Range);
#endif


#endif /*__M24LRXX_COMMAND_H */


