/**
  ******************************************************************************
  * @file iso15693_command.h
  * @brief Header for the iso15693_command.c file 
  * @author STMicroelectronics - MMY Application Team
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
	
	
#ifndef __ISO15693_COMMAND_H
#define __ISO15693_COMMAND_H


// Includes ------------------------------------------------------------------



//#define USE_FULL_ISO15693_COMMANDS // uncomment to access the write multiple blocks commands, as a few contactless tag support this command, it's not compiled to save memory space 


// Lower level
#include "iso15693_structure.h"
#include "cr95hf_command.h"



// Constants for iso15693 layer
// Note that ICRef constants related to the product identification are gathered in miscellaneous.h

// COnstants for tag command code
#define ISO15693_CMD_INVENTORY 									0x01 
#define ISO15693_CMD_STAY_QUIET									0x02 
#define ISO15693_CMD_READ_SINGLE_BLOCK 					0x20
#define ISO15693_CMD_WRITE_SINGLE_BLOCK					0x21	
#define ISO15693_CMD_LOCK_BLOCK									0x22
#define ISO15693_CMD_READ_MULTIPLE_BLOCKS				0x23
#define ISO15693_CMD_WRITE_MULTIPLE_BLOCKS			0x24
#define ISO15693_CMD_SELECT											0x25
#define ISO15693_CMD_RESET_TO_READY							0x26
#define ISO15693_CMD_WRITE_AFI									0x27
#define ISO15693_CMD_LOCK_AFI										0x28
#define ISO15693_CMD_WRITE_DSFID								0x29
#define ISO15693_CMD_LOCK_DSFID									0x2A
#define ISO15693_CMD_GET_SYSTEM_INFO						0x2B
#define ISO15693_CMD_GET_MULTIPLE_BLOCKS_STATUS	0x2C

//Constants for counters length and cursors
#define ISO15693_ICREF_SHIFT				 			0x02	
#define ISO15693_ICREF_CURSOR_IN_UID 			0x05

#define ISO15693_UID_CURSOR_CR95HF_RESPONSE_DATA	0x02
#define ISO15693_INVENTORY_16_SLOTS_COUNTER				0x0F// equal to 15 because first slot is right after the command sending
#define ISO15693_DEFAULT_BLOCK_SIZE								0x04
#define ISO15693_DEFAULT_BLOCK_PLUS_STATUS_SIZE		0x05
#define ISO15693_MAX_SIZE_BLOCK_WRITE_CMD					0x1F // for write multiple blocks limited to 32 blocks (31 after the first one -> 1024 bits with 4 bytes per block 
#define ISO15693_MAX_SIZE_BLOCK_READ_CMD					0x1F // limited to 32 block (31 after the first one)
#define ISO15693_UID_SIZE_BITS										0x40 // UID sizes 64 bits

#define ISO15693_MANUFACTURER_CURSOR			0x06
#define ISO15693_ERROR_CODE_CURSOR				0x01
#define ISO15693_INFORMATION_FLAG_CURSOR	0x01
#define ISO15693_MEMSIZE_3BYTES_BLOCK_SIZE_CURSOR 0x02
#define ISO15693_MEMSIZE_2BYTES_BLOCK_SIZE_CURSOR 0x01

// Length of information parameters in read commands (response flags, CRC , CRC check) 
#define ISO15693_LENGTH_PARAMETERS_READ_RESPONSE 	0x04
// Length of mandatory information received for a get system info command
#define ISO15693_LENGTH_FIX_PARAMETERS_GET_SYS_INFO_RESPONSE 	0x0D
// Response flags (1byte), Information flags (1)
// UID (8), CRC (2), Ctrl byte added by the CR95HF (1) 

// Size of a read multiple Block with ISO15693_MAX_SIZE_BLOCK_READ_CMD and status
// the size contains frame data response flags, CRC, control byte + number of byte read (here 32 block of 4 bytes ie. 128bytes + the status if expected 32 bytes  
#define ISO15693_READ_MULITPLE_BLOCK_SIZE_ISO15693_MAX_SIZE_BLOCK_READ_CMD  (SIZE_1KBITS_BYTES+ISO15693_LENGTH_PARAMETERS_READ_RESPONSE+ISO15693_MAX_SIZE_BLOCK_READ_CMD+1)


#define ISO15693_ERROR_FRAME_SIZE 								0x05 // contactless tag answer when an error occured (CRC, and CRC checked addred by CR95HF included)

// Particular values

// STMicroelectronics MANUFACTURER CODE
#define ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE		0x02


// Constants for mask
#define ISO15693_CRC_MASK											0x02
#define ISO15693_COLLISION_DETECTION_MASK			0x01
#define ISO15693_REQUEST_FLAGS_BITS_SELECT_ADDRESS_MASK	0x30	


// Constants for returning value
#define ISO15693_SUCCESS_CODE										0x00
#define ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS	0x02
#define ISO15693_ERROR_CODE											0x01
#define ISO15693_BLOCK_SIZE_ERROR								0x00
#define ISO15693_ERROR_CRC  										0xCC // arbritrary defined constant
#define ISO15693_ERROR_COLLISION_DETECTED  			0xCD // arbritrary defined constant
#define ISO15693_ERROR_COLLISION_CRC  					0xCF // arbritrary defined constant

// error code
/* Informative uncomment to use the constants
#define ISO15693_ERROR_COMMAND_NOT_SUPPORTED											0x01
#define ISO15693_ERROR_COMMAND_NOT_RECOGNISED											0x02
#define ISO15693_ERROR_COMMAND_OPTION_NOT_SUPPORTED 							0x03
#define ISO15693_ERROR_NO_INFORMATION_OR_CODE_NOT_SUPPORTED				0x0F
#define ISO15693_ERROR_SPECIFIED_BLOCK_NOT_AVAILABLE							0x10
#define ISO15693_ERROR_SPECIFIED_BLOCK_ALREADY_LOCKED							0x11
#define ISO15693_ERROR_SPECIFIED_BLOCK_LOCKED											0x12
#define ISO15693_ERROR_SPECIFIED_BLOCK_UNSUCCESSFULLY_PROGRAMMED 	0x13
#define ISO15693_ERROR_SPECIFIED_BLOCK_UNSUCCESSFULLY_LOCKED    	0x14
*/

// CRC 16 constants
#define ISO15693_PRELOADCRC16 					0xFFFF 
#define ISO15693_POLYCRC16 							0x8408 
#define ISO15693_MASKCRC16 							0x0001
#define ISO15693_RESIDUECRC16 					0xF0B8
#define ISO15693_RESULTOKCRC16						0x00
#define ISO15693_ERRORCODE_GENERICCRC16		0x01


//RequestFlag-------------------------------------------------------------

#define ISO15693_FLAG_TEST_BIT3					0x04
#define ISO15693_AFI_TEST_BIT5					0x10

#define ISO15693_BIT1_SINGLE_SUBCARRIER					0x00
#define ISO15693_BIT1_DUAL_SUBCARRIERS					0x01
#define ISO15693_BIT2_LOW_DATARATE							0x00
#define ISO15693_BIT2_HIGH_DATARATE							0x02
#define ISO15693_BIT3_INVENTORY_FLAG_RESET		  0x00
#define ISO15693_BIT3_INVENTORY_FLAG_SET				0x04
#define ISO15693_BIT4_NO_PROTOCOL_EXTENSION			0x00
#define ISO15693_BIT4_PROTOCOL_EXTENSION				0x08

// BIT3 (INVENTORY FLAG) IS RESET
#define ISO15693_BIT3RESET_BIT5_REQUEST_TAG_NOT_SELECTED	0x00
#define ISO15693_BIT3RESET_BIT5_REQUEST_TAG_SELECTED			0x10
#define ISO15693_BIT3RESET_BIT6_REQUEST_NOT_ADDRESSED			0x00
#define ISO15693_BIT3RESET_BIT6_REQUEST_ADDRESSED					0x20
#define ISO15693_BIT3RESET_BIT7_OPTIONFLAG_RESET					0x00 
#define ISO15693_BIT3RESET_BIT7_OPTIONFLAG_SET						0x40 
#define ISO15693_BIT3RESET_BIT8_RFU												0x00

// INVENTORY FLAG IS SET
#define ISO15693_BIT3SET_BIT5_AFI_NOT_PRESENT						0x00
#define ISO15693_BIT3SET_BIT5_AFI_PRESENT								0x10
#define ISO15693_BIT3SET_BIT6_16_SLOTS									0x00
#define ISO15693_BIT3SET_BIT6_1_SLOT										0x20
#define ISO15693_BIT3SET_BIT7_OPTIONFLAG_RESET					0x00
#define ISO15693_BIT3SET_BIT7_OPTIONFLAG_SET						0x40 
#define ISO15693_BIT3SET_BIT8_RFU												0x00

#define ISO15693_BIT8_RFU_SET_MASK											0x80

// Parameters value for getting info functions
#define ISO15693_SINGLE_SUBCARRIER					0x00
#define ISO15693_DUAL_SUBCARRIERS						0x01
#define ISO15693_LOW_DATARATE								0x00
#define ISO15693_HIGH_DATARATE							0x02
#define ISO15693_INVENTORY_FLAG_RESET				0x00
#define ISO15693_INVENTORY_FLAG_SET					0x04
#define ISO15693_NO_PROTOCOL_EXTENSION			0x00
#define ISO15693_PROTOCOL_EXTENSION					0x08

// INVENTRORYFLAG=0
#define ISO15693_SELECT_FLAG_RESET					0x00
#define ISO15693_SELECT_FLAG_SET						0x10
#define ISO15693_ADDRESS_FLAG_RESET					0x00
#define ISO15693_ADDRESS_FLAG_SET						0x20

// INVENTRORYFLAG=1
#define ISO15693_AFI_NOT_PRESENT						0x00
#define ISO15693_AFI_PRESENT								0x01
#define ISO15693_16_SLOTS										0x00
#define ISO15693_1_SLOT											0x01

#define ISO15693_OPTION_FLAG_SET						0x40
#define ISO15693_OPTION_FLAG_RESET					0x00
#define ISO15693_BIT8_RFU_RESET							0x00
#define ISO15693_BIT8_RFU_SET								0x80
#define ISO15693_ERROR_INVALID_FLAG					0xFF

//-------------------------------------------------------------------
//Response Flag
#define ISO15693_RESPONSE_FLAGS_ERRORFLAG_MASK			0x01

//-------------------------------------------------------------------
// Information Flag
#define ISO15693_INFORMATION_FLAGS_DSFID_MASK			0x01
#define ISO15693_INFORMATION_FLAGS_AFI_MASK				0x02
#define ISO15693_INFORMATION_FLAGS_MEMSIZE_MASK		0x04
#define ISO15693_INFORMATION_FLAGS_ICREF_MASK		0x08

//UID MSB
#define ISO15693_UID_MSB	0xE0  // To check MSB into EEPROM

// Exported macro functions --------------------------------------------------------
                                 


u8 Scan15693Tag (void);


// Command functions --------------------------------------------------------

u8 ISO15693_Inventory(ISO15693_Tag* MyTag,const u8 Request_flags,const  u8 Tag_AFI, 
const u8 MaskLength,const u8* Mask, 
u8* Inventory_16_slots_Nb_UID_Retrieved);

u8 ISO15693_Stay_Quiet(const u8 Request_flags,const  u8* Tag_UID);

u8 ISO15693_Read_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Block_number);

u8 ISO15693_Write_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Block_number,const u8 Block_Length, const u8* Data);

u8 ISO15693_Lock_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Block_number);

u8 ISO15693_Read_Multiple_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 First_Block_number, const u8 Number_of_Blocks);

#ifdef USE_FULL_ISO15693_COMMANDS
	u8 ISO15693_Write_Multiple_Block(ISO15693_Tag* MyTag, const u8 Request_flags,const  u8* Tag_UID, const u8 First_Block_number, const u8 Number_of_Blocks, const u8 Block_Length);
#endif 

u8 ISO15693_Select(ISO15693_Tag* MyTag,const u8 Request_flags, const u8* Tag_UID);

u8 ISO15693_Reset_to_Ready(ISO15693_Tag* MyTag,const u8 Request_flags, const u8* Tag_UID);

u8 ISO15693_Write_AFI(ISO15693_Tag* MyTag,  const u8 Request_flags,const  u8* Tag_UID, const u8 Tag_AFI);

u8 ISO15693_Lock_AFI(ISO15693_Tag* MyTag, const u8 Request_flags,const  u8* Tag_UID);

u8 ISO15693_Write_DSFID(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Tag_DSFID);

u8 ISO15693_Lock_DSFID(ISO15693_Tag* MyTag,const  u8 Request_flags,const  u8* Tag_UID);

u8 ISO15693_Get_System_Info(ISO15693_Tag* MyTag, const u8 Request_flags,const u8* Tag_UID);

u8 ISO15693_Get_Multiple_Blocks_Security_Status(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 First_Block_number, const u8 Number_of_Blocks);

u8 ISO15693_Custom_Commands(ISO15693_Tag* MyTag, u8 Request_flags, u8 Custom, u8 IC_Mfg_Code, u8 Parameters_Length, u8* parameters);

// Advanced functions -------------------------------------------------------
#ifdef USE_EXTERN_EEPROM_FUNCTION
u8 ISO15693_Read_Blocks_Write_in_Ext_EEPROM(ISO15693_Tag* MyTag,const  u8 Request_flags,const u8* Tag_UID,const u16 First_Block_Number, const u16 Number_of_Blocks);
#endif

//--------------------------------------------------------------------
// Support functions ---------------------------------
// Structure initialization functions ----------------

static void ISO15693_Init_TagStructure_Generic(ISO15693_Tag* MyTag);

static void ISO15693_Init_TagStructure_Inventory_cmd(ISO15693_Tag* MyTag);

static void ISO15693_Init_TagStructure_GetSysInfo_cmd(ISO15693_Tag* MyTag);

static void ISO15693_Init_TagStructure_Read_Multiple_Blocks_cmd(ISO15693_Tag* MyTag);

static void ISO15693_Init_TagStructure_Get_Multiple_Block_Status_cmd(ISO15693_Tag* MyTag);

static void ISO15693_Init_TagStructure_MemSize(ISO15693_Tag* MyTag);


//----------------------------------------------------
//Add data to command --------------------------------

void ISO15693_Add_Request_Flags_Command_code_to_command(CR95HF_CommandSending* Cmd, 
u8* Cptr_ToIncrement, const u8 Request_flags ,const u8 Command_Code);

void ISO15693_Add_Mask_to_command(CR95HF_CommandSending* Cmd, 
u8* Cptr_ToIncrement,const u8 MaskLength, const u8* Mask);

void ISO15693_Add_UID_to_command(CR95HF_CommandSending* Cmd, 
u8* Cptr_ToIncrement, const u8* Tag_UID ,const u8 Request_flags);

//-------------------------------------------------------
// Is ? functions----------------------------------------

u8 ISO15693_Is_ResponseFlags_noError_Detected(const u8 ISO15693_Response_flags);

u8 ISO15693_Is_Request_Flags_ok(const u8 Request_flags, const u8 ISO15693_cmd_code);

u8 ISO15693_Is_Collision_Flag_Set( CR95HF_ResponseReceiving* Response);

u8 ISO15693_Is_CRC_Flag_Set(CR95HF_ResponseReceiving* Response);

u8 ISO15693_Is_CorrectCRC16_Check( CR95HF_ResponseReceiving* Response);

u8 ISO15693_Is_TagConstructor_STM(const ISO15693_Tag* MyTag);

u8 ISO15693_Is_Memory_programming_function(const u8 ISO15693_cmd_code);

u8 ISO15693_Is_DSFID_Present_Infoflag( const u8 Information_flags);

u8 ISO15693_Is_AFI_Present_Infoflag(const  u8 Information_flags);

u8 ISO15693_Is_MemSize_Present_Infoflag( const u8 Information_flags);

u8 ISO15693_Is_ICRef_Present_Infoflag( const u8 Information_flags);

//-------------------------------------------------------
// Get parameter functions-------------------------------

u8 ISO15693_Get_Subcarrier_Flag(const u8 Request_flags);

u8 ISO15693_Get_DataRate_Flag(const u8 Request_flags);

u8 ISO15693_Get_Inventory_Flag(const u8 Request_flags);

u8 ISO15693_Get_ProtocolExtension_Flag(const u8 Request_flags);

u8 ISO15693_Get_Select_Flag(const u8 Request_flags);

u8 ISO15693_Get_Address_Flag(const u8 Request_flags);

u8 ISO15693_Get_AFI_Flag(const u8 Request_flags);

u8 ISO15693_Get_Slots_Flag(const u8 Request_flags);

u8 ISO15693_Get_Option_Flag(const u8 Request_flags);

u8 ISO15693_Get_RFU_Flag(const u8 Request_flags);

u8 ISO15693_Get_SubCarrier_parameter_CR95HF(const CR95HF_Protocol_Config* Protocol_Settings);

u8 ISO15693_Get_Modulation_parameter_CR95HF(const CR95HF_Protocol_Config* Protocol_Settings);

u8 ISO15693_Get_Delay_parameter_CR95HF(const CR95HF_Protocol_Config* Protocol_Settings);

u8 ISO15693_Get_DataRate_parameter_CR95HF(const CR95HF_Protocol_Config* Protocol_Settings);

u8 ISO15693_Get_NumberofByte_MemSize_using_ICREF(const u8 ICRef);

u8 ISO15693_Get_Size_of_Block_using_ICRef(const u8 ICRef);

u8 ISO15693_Get_Size_of_Block_using_MemSize(const ISO15693_Tag* MyTag);

u16 ISO15693_Get_MemSize_kbits(const u8* MemSize, const u8 Number_of_bytes_MemSize);

u32 ISO15693_Get_MemSize_bits(const u8* MemSize, const u8 Number_of_bytes_MemSize);

u8 ISO15693_Get_UID_from_EEPROM(u8* UID_Tag,  u8 UID_number);

//-------------------------------------------------------
// CRC functions-----------------------------------------

int16_t ISO15693_CRC16 (const u8* DataIn,const u8 NbByte) ;

u8 ISO15693_IsCorrectCRC16Residue (const u8 *DataIn,const u8 Length);

int16_t ISO15693_CRC16_EEPROM (const u8 NbByte) ;

u8 ISO15693_IsCorrectCRC16Residue_EEPROM (const u8 Length);

//-------------------------------------------------------
// Interpretation functions------------------------------

u8 ISO156693_Check_Data_Received_Start_Process(ISO15693_Tag* MyTag,const CR95HF_ResponseReceiving *Response,  u8* Cursor_in_response_data );

void ISO15693_Fill_in_Tag_Structure_ErrorFlag_set(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving *Response);

u8  ISO15693_Read_Answer_Format_Response_Flags(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving *Response,const u8 Status_cr95hf);
//-------------------------------------------------------
// Command support functions-----------------------------

u8 ISO15693_Send_Receive_Inventory_cmd(ISO15693_Tag* MyTag, CR95HF_ResponseReceiving* Response,CR95HF_CommandSending* Cmd, u8 Request_flags, u8* status_cr95hf, u8* Inventory_16_slots_Nb_UID_Retrieved);

static u8 ISO15693_Send_Receive_Inventory_16_slots(CR95HF_ResponseReceiving* Response,CR95HF_CommandSending* Cmd, u8* Inventory_16_slots_Nb_UID_Retrieved);

u8 ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(CR95HF_ResponseReceiving* Response,CR95HF_CommandSending* Cmd, const u8 Request_flags);

//#ifdef USE_FULL_ISO15693_COMMANDS
	static u8 ISO15693_Send_Multiple_Blocks_cmd(CR95HF_ResponseReceiving* Response,CR95HF_CommandSending* Cmd, const u8 Request_flags, const u8 Number_of_Bytes_to_Transmit);

static void ISO15693_Send_Multiple_Blocks_CR95HF_level(CR95HF_CommandSending* Cmd,  u8 Number_of_Bytes_to_Transmit);
//#endif

static u8 ISO15693_SendEOF(CR95HF_ResponseReceiving* Response);

u8 ISO15693_Receive_Multiple_Blocks_cmd(CR95HF_ResponseReceiving* Response);

static 	ISO15693_Receive_Multiple_Blocks_Store_into_EEPROM(CR95HF_ResponseReceiving* Response);

//-------------------------------------------------------
// Fill in tag structure functions ----------------------

 u8 ISO156693_Fill_in_TagStructure_Inventory_cmd(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, u8 CR95HF_command_status );

static void ISO156693_Fill_in_UID_Array_Inventory_16_slots_cmd( 
u8* Inventory_16_slots_Nb_UID_Retrieved,CR95HF_ResponseReceiving* Response);

u8 ISO156693_Fill_in_TagStructure_Read_Single_Block_cmd(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, const u8 CR95HF_command_status, const u8 Request_flags  );

u8 ISO156693_Fill_in_TagStructure_Read_Multiple_Blocks_cmd(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, const u8 CR95HF_command_status, const u8 Number_of_Blocks );

static u8 ISO156693_Fill_in_TagStructure_GetSysInfo_cmd(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, const u8 CR95HF_command_status );

u8 ISO156693_Fill_in_TagStructure_Get_Multiple_Block_Status_cmd(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, const u8 CR95HF_command_status,const  u8 Number_of_Blocks );

static u8 ISO156693_Fill_in_TagStructure_Custom_cmd(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, const u8 CR95HF_command_status );


//--------------------------------------------------------------------
// For data ----------------------------------------------------------
 
void ISO15693_Retrieve_UID(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, u8* Cursor_in_response_data);

static void ISO15693_Retrieve_UID_16_slots(u8 Inventory_16_slots_Nb_UID_Retrieved ,CR95HF_ResponseReceiving* Response);

void ISO15693_Retrieve_ICRef(ISO15693_Tag* MyTag);

static void ISO15693_Retrieve_Data_With_Information_Flags(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, const u8 Information_flags, u8* Cursor_in_response_data);

//--------------------------------------------------------------------
// Others ------------------------------------------------------------

static u8 ISO15693_Compute_Number_of_bytes_MemSize_using_GetSysInfo_Frame(const u8 Information_Flags,const u8 Response_data_length);

#endif /*__ISO15693_COMMAND_H */

